@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,6 +35,7 @@
 /* That one is necessary for defines of OPTION_NO_FOREIGN_KEY_CHECKS etc */
 #include "mysql_priv.h" 
 #include "log_event.h"
+#include "sql_common.h"
 
 #define BIN_LOG_HEADER_SIZE	4
 #define PROBE_HEADER_LEN	(EVENT_LEN_OFFSET+4)
@@ -93,8 +93,10 @@ static bool stop_passed= 0;
 */
 Format_description_log_event* description_event; 
 
-static int dump_local_log_entries(const char* logname);
-static int dump_remote_log_entries(const char* logname);
+static int dump_local_log_entries(PRINT_EVENT_INFO *print_event_info,
+                                  const char* logname);
+static int dump_remote_log_entries(PRINT_EVENT_INFO *print_event_info,
+                                   const char* logname);
 static int dump_log_entries(const char* logname);
 static int dump_remote_file(NET* net, const char* fname);
 static void die(const char* fmt, ...);
@@ -949,8 +951,22 @@ static MYSQL* safe_connect()
 
 static int dump_log_entries(const char* logname)
 {
-  return (remote_opt ? dump_remote_log_entries(logname) :
-          dump_local_log_entries(logname));
+  int rc;
+  PRINT_EVENT_INFO print_event_info;
+  /*
+     Set safe delimiter, to dump things
+     like CREATE PROCEDURE safely
+  */
+  fprintf(result_file, "DELIMITER /*!*/;\n");
+  strcpy(print_event_info.delimiter, "/*!*/;");
+
+  rc= (remote_opt ? dump_remote_log_entries(&print_event_info, logname) :
+       dump_local_log_entries(&print_event_info, logname));
+
+  /* Set delimiter back to semicolon */
+  fprintf(result_file, "DELIMITER ;\n");
+  strcpy(print_event_info.delimiter, ";");
+  return rc;
 }
 
 
@@ -1015,11 +1031,11 @@ static int check_master_version(MYSQL* mysql,
 }
 
 
-static int dump_remote_log_entries(const char* logname)
+static int dump_remote_log_entries(PRINT_EVENT_INFO *print_event_info,
+                                   const char* logname)
 
 {
   char buf[128];
-  PRINT_EVENT_INFO print_event_info;
   ulong len;
   uint logname_len;
   NET* net;
@@ -1077,7 +1093,7 @@ could be out of memory");
     const char *error_msg;
     Log_event *ev;
 
-    len = net_safe_read(mysql);
+    len= cli_safe_read(mysql);
     if (len == packet_error)
     {
       fprintf(stderr, "Got error reading packet from server: %s\n",
@@ -1087,7 +1103,7 @@ could be out of memory");
     }
     if (len < 8 && net->read_pos[0] == 254)
       break; // end of data
-    DBUG_PRINT("info",( "len= %u, net->read_pos[5] = %d\n",
+    DBUG_PRINT("info",( "len: %lu, net->read_pos[5]: %d\n",
 			len, net->read_pos[5]));
     if (!(ev= Log_event::read_log_event((const char*) net->read_pos + 1 ,
                                         len - 1, &error_msg,
@@ -1142,7 +1158,7 @@ could be out of memory");
           len= 1; // fake Rotate, so don't increment old_off
         }
       }
-      if ((error= process_event(&print_event_info, ev, old_off)))
+      if ((error= process_event(print_event_info, ev, old_off)))
       {
 	error= ((error < 0) ? 0 : 1);
         goto err;
@@ -1161,7 +1177,7 @@ could be out of memory");
         goto err;
       }
       
-      if ((error= process_event(&print_event_info, ev, old_off)))
+      if ((error= process_event(print_event_info, ev, old_off)))
       {
  	my_close(file,MYF(MY_WME));
 	error= ((error < 0) ? 0 : 1);
@@ -1287,11 +1303,11 @@ at offset %lu ; this could be a log format error or read error",
 }
 
 
-static int dump_local_log_entries(const char* logname)
+static int dump_local_log_entries(PRINT_EVENT_INFO *print_event_info,
+                                  const char* logname)
 {
   File fd = -1;
   IO_CACHE cache,*file= &cache;
-  PRINT_EVENT_INFO print_event_info;
   byte tmp_buff[BIN_LOG_HEADER_SIZE];
   int error= 0;
 
@@ -1309,6 +1325,25 @@ static int dump_local_log_entries(const char* logname)
   }
   else // reading from stdin;
   {
+    /*
+      Bug fix: #23735
+      Author: Chuck Bell
+      Description:
+        Windows opens stdin in text mode by default. Certain characters
+        such as CTRL-Z are interpeted as events and the read() method
+        will stop. CTRL-Z is the EOF marker in Windows. to get past this
+        you have to open stdin in binary mode. Setmode() is used to set
+        stdin in binary mode. Errors on setting this mode result in
+        halting the function and printing an error message to stderr.
+    */
+#if defined (__WIN__) || (_WIN64)
+    if (_setmode(fileno(stdin), O_BINARY) == -1)
+    {
+       fprintf(stderr, "Could not set binary mode on stdin.\n");
+       return 1;
+    }
+#endif
+
     if (init_io_cache(file, fileno(stdin), 0, READ_CACHE, (my_off_t) 0,
 		      0, MYF(MY_WME | MY_NABP | MY_DONT_CHECK_FILESIZE)))
       return 1;
@@ -1363,7 +1398,7 @@ static int dump_local_log_entries(const char* logname)
       // file->error == 0 means EOF, that's OK, we break in this case
       break;
     }
-    if ((error= process_event(&print_event_info, ev, old_off)))
+    if ((error= process_event(print_event_info, ev, old_off)))
     {
       if (error < 0)
         error= 0;
@@ -1489,14 +1524,13 @@ int main(int argc, char** argv)
   the server
 */
 
-#ifdef __WIN__
 #include "my_decimal.h"
 #include "decimal.c"
+
+#if defined(__WIN__) && !defined(CMAKE_BUILD)
 #include "my_decimal.cpp"
 #include "log_event.cpp"
 #else
-#include "my_decimal.h"
-#include "decimal.c"
 #include "my_decimal.cc"
 #include "log_event.cc"
 #endif
