@@ -1,23 +1,20 @@
-/* integer.cpp                                
- *
- * Copyright (C) 2003 Sawtooth Consulting Ltd.
- *
- * This file is part of yaSSL.
- *
- * yaSSL is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * yaSSL is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- */
+/*
+   Copyright (C) 2000-2007 MySQL AB
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; see the file COPYING. If not, write to the
+   Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+   MA  02110-1301  USA.
+*/
 
 
 
@@ -34,29 +31,21 @@
     #include <c_asm.h>  // for asm overflow assembly
 #endif
 
+#if defined(_M_X64) || defined(_M_IA64)
+    #include <intrin.h> 
+#pragma intrinsic(_umul128)
+#endif
 
-// 64bit multiply overflow intrinsic
-#if defined(_MSC_VER) && defined(_WIN64) && !defined(__INTEL_COMPILER) && \
-   !defined(TAOCRYPT_NATIVE_DWORD_AVAILABLE)
-    #ifdef __ia64__
-        #define myUMULH __UMULH
-    #elif  __x86_64__
-        #define myUMULH __umulh
-    #else
-        #error unknown 64bit windows
-    #endif
 
-extern "C" word myUMULH(word, word); 
-
-#pragma intrinsic (myUMULH)
+#ifdef __GNUC__
+    #include <signal.h>
+    #include <setjmp.h>
 #endif
 
 
 #ifdef SSE2_INTRINSICS_AVAILABLE
     #ifdef __GNUC__
         #include <xmmintrin.h>
-        #include <signal.h>
-        #include <setjmp.h>
         #ifdef TAOCRYPT_MEMALIGN_AVAILABLE
             #include <malloc.h>
         #else
@@ -81,7 +70,7 @@ namespace TaoCrypt {
 #ifdef SSE2_INTRINSICS_AVAILABLE
 
 template <class T>
-CPP_TYPENAME AllocatorBase<T>::pointer AlignedAllocator<T>::allocate(
+CPP_TYPENAME AlignedAllocator<T>::pointer AlignedAllocator<T>::allocate(
                                            size_type n, const void *)
 {
     CheckSize(n);
@@ -177,9 +166,8 @@ DWord() {}
         #ifdef TAOCRYPT_NATIVE_DWORD_AVAILABLE
             r.whole_ = (dword)a * b;
 
-        #elif defined(_MSC_VER)
-            r.halfs_.low = a*b;
-            r.halfs_.high = myUMULH(a,b);
+        #elif defined(_M_X64) || defined(_M_IA64)
+            r.halfs_.low = _umul128(a, b, &r.halfs_.high);
 
         #elif defined(__alpha__)
             r.halfs_.low = a*b;
@@ -565,24 +553,29 @@ static word AtomicInverseModPower2(word A)
 class Portable
 {
 public:
-    static word Add(word *C, const word *A, const word *B, unsigned int N);
-    static word Subtract(word *C, const word *A, const word*B, unsigned int N);
+    static word TAOCRYPT_CDECL Add(word *C, const word *A, const word *B,
+                                   unsigned int N);
+    static word TAOCRYPT_CDECL Subtract(word *C, const word *A, const word*B,
+                                        unsigned int N);
+    static void TAOCRYPT_CDECL Multiply2(word *C, const word *A, const word *B);
+    static word TAOCRYPT_CDECL Multiply2Add(word *C,
+                                            const word *A, const word *B);
+    static void TAOCRYPT_CDECL Multiply4(word *C, const word *A, const word *B);
+    static void TAOCRYPT_CDECL Multiply8(word *C, const word *A, const word *B);
+    static unsigned int TAOCRYPT_CDECL MultiplyRecursionLimit() {return 8;}
 
-    static void Multiply2(word *C, const word *A, const word *B);
-    static word Multiply2Add(word *C, const word *A, const word *B);
-    static void Multiply4(word *C, const word *A, const word *B);
-    static void Multiply8(word *C, const word *A, const word *B);
-    static unsigned int MultiplyRecursionLimit() {return 8;}
+    static void TAOCRYPT_CDECL Multiply2Bottom(word *C, const word *A,
+                                               const word *B);
+    static void TAOCRYPT_CDECL Multiply4Bottom(word *C, const word *A,
+                                               const word *B);
+    static void TAOCRYPT_CDECL Multiply8Bottom(word *C, const word *A,
+                                               const word *B);
+    static unsigned int TAOCRYPT_CDECL MultiplyBottomRecursionLimit(){return 8;}
 
-    static void Multiply2Bottom(word *C, const word *A, const word *B);
-    static void Multiply4Bottom(word *C, const word *A, const word *B);
-    static void Multiply8Bottom(word *C, const word *A, const word *B);
-    static unsigned int MultiplyBottomRecursionLimit() {return 8;}
-
-    static void Square2(word *R, const word *A);
-    static void Square4(word *R, const word *A);
-    static void Square8(word *R, const word *A) {assert(false);}
-    static unsigned int SquareRecursionLimit() {return 4;}
+    static void TAOCRYPT_CDECL Square2(word *R, const word *A);
+    static void TAOCRYPT_CDECL Square4(word *R, const word *A);
+    static void TAOCRYPT_CDECL Square8(word *R, const word *A) {assert(false);}
+    static unsigned int TAOCRYPT_CDECL SquareRecursionLimit() {return 4;}
 };
 
 word Portable::Add(word *C, const word *A, const word *B, unsigned int N)
@@ -1011,44 +1004,20 @@ void Portable::Multiply8Bottom(word *R, const word *A, const word *B)
 
 // ************** x86 feature detection ***************
 
-static bool s_sse2Enabled = true;
-
-static void CpuId(word32 input, word32 *output)
-{
-#ifdef __GNUC__
-    __asm__
-    (
-        // save ebx in case -fPIC is being used
-        "push %%ebx; cpuid; mov %%ebx, %%edi; pop %%ebx"
-        : "=a" (output[0]), "=D" (output[1]), "=c" (output[2]), "=d"(output[3])
-        : "a" (input)
-    );
-#else
-    __asm
-    {
-        mov eax, input
-        cpuid
-        mov edi, output
-        mov [edi], eax
-        mov [edi+4], ebx
-        mov [edi+8], ecx
-        mov [edi+12], edx
-    }
-#endif
-}
 
 #ifdef SSE2_INTRINSICS_AVAILABLE
+
 #ifndef _MSC_VER
-static jmp_buf s_env;
-static void SigIllHandler(int)
-{
+    static jmp_buf s_env;
+    static void SigIllHandler(int)
+    {
     longjmp(s_env, 1);
-}
+    }
 #endif
 
 static bool HasSSE2()
 {
-    if (!s_sse2Enabled)
+    if (!IsPentium())
         return false;
 
     word32 cpuid[4];
@@ -1077,23 +1046,22 @@ static bool HasSSE2()
     if (setjmp(s_env))
         result = false;
     else
-        __asm __volatile ("xorps %xmm0, %xmm0");
+        __asm __volatile ("xorpd %xmm0, %xmm0");
 
     signal(SIGILL, oldHandler);
     return result;
 #endif
 }
-#endif
+#endif // SSE2_INTRINSICS_AVAILABLE
+
 
 static bool IsP4()
 {
-    word32 cpuid[4];
-
-    CpuId(0, cpuid);
-    mySTL::swap(cpuid[2], cpuid[3]);
-    if (memcmp(cpuid+1, "GenuineIntel", 12) != 0)
+    if (!IsPentium())
         return false;
 
+    word32 cpuid[4];
+
     CpuId(1, cpuid);
     return ((cpuid[0] >> 8) & 0xf) == 0xf;
 }
@@ -1143,7 +1111,12 @@ static PMul s_pMul4, s_pMul8, s_pMul8B;
 
 static void SetPentiumFunctionPointers()
 {
-    if (IsP4())
+    if (!IsPentium())
+    {   
+        s_pAdd = &Portable::Add;
+        s_pSub = &Portable::Subtract;
+    }
+    else if (IsP4())
     {
         s_pAdd = &P4Optimized::Add;
         s_pSub = &P4Optimized::Subtract;
@@ -1155,7 +1128,13 @@ static void SetPentiumFunctionPointers()
     }
 
 #ifdef SSE2_INTRINSICS_AVAILABLE
-    if (HasSSE2())
+    if (!IsPentium()) 
+    {
+        s_pMul4 = &Portable::Multiply4;
+        s_pMul8 = &Portable::Multiply8;
+        s_pMul8B = &Portable::Multiply8Bottom;
+    }
+    else if (HasSSE2())
     {
         s_pMul4 = &P4Optimized::Multiply4;
         s_pMul8 = &P4Optimized::Multiply8;
@@ -1173,11 +1152,6 @@ static void SetPentiumFunctionPointers()
 static const char s_RunAtStartupSetPentiumFunctionPointers =
     (SetPentiumFunctionPointers(), 0);
 
-void DisableSSE2()
-{
-    s_sse2Enabled = false;
-    SetPentiumFunctionPointers();
-}
 
 class LowLevel : public PentiumOptimized
 {
@@ -2380,8 +2354,8 @@ void AsymmetricMultiply(word *R, word *T, const word *A, unsigned int NA,
 
     if (NA > NB)
     {
-        mySTL::swap(A, B);
-        mySTL::swap(NA, NB);
+        STL::swap(A, B);
+        STL::swap(NA, NB);
     }
 
     assert(NB % NA == 0);
@@ -2517,8 +2491,8 @@ unsigned int AlmostInverse(word *R, word *T, const word *A, unsigned int NA,
 
         if (Compare(f, g, fgLen)==-1)
         {
-            mySTL::swap(f, g);
-            mySTL::swap(b, c);
+            STL::swap(f, g);
+            STL::swap(b, c);
             s++;
         }
 
@@ -3158,7 +3132,7 @@ signed long Integer::ConvertToLong() const
 void Integer::Swap(Integer& a)
 {
     reg_.Swap(a.reg_);
-    mySTL::swap(sign_, a.sign_);
+    STL::swap(sign_, a.sign_);
 }
 
 
@@ -3980,6 +3954,9 @@ Integer CRT(const Integer &xp, const Integer &p, const Integer &xq,
 template hword DivideThreeWordsByTwo<hword, Word>(hword*, hword, hword, Word*);
 #endif
 template word DivideThreeWordsByTwo<word, DWord>(word*, word, word, DWord*);
+#ifdef SSE2_INTRINSICS_AVAILABLE
+template class AlignedAllocator<word>;
+#endif
 #endif
 
 
