@@ -5,8 +5,8 @@
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   License as published by the Free Software Foundation; version 2
+   of the License.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -463,6 +463,9 @@ typedef unsigned long long os_off_t;
 /* Define if you have the getwd function.  */
 #define HAVE_GETWD 1
 
+/* Define to 1 if you have the `gmtime_r' function. */
+#define HAVE_GMTIME_R 1
+
 /* Define if you have the index function.  */
 #define HAVE_INDEX 1
 
