@@ -616,7 +616,7 @@ btr_page_get_father_for_rec(
 		fputs(
 "InnoDB: You should dump + drop + reimport the table to fix the\n"
 "InnoDB: corruption. If the crash happens at the database startup, see\n"
-"InnoDB: http://dev.mysql.com/doc/mysql/en/Forcing_recovery.html about\n"
+"InnoDB: http://dev.mysql.com/doc/refman/5.0/en/forcing-recovery.html about\n"
 "InnoDB: forcing recovery. Then dump + drop + reimport.\n", stderr);
 	}
 
