@@ -46,7 +46,7 @@ Note that contary to what we planned in the 1990's, there will only be one
 insert buffer tree, and that is in the system tablespace of InnoDB.
 
 1. The first field is the space id.
-2. The second field is a one-byte marker which differentiates records from
+2. The second field is a one-byte marker (0) which differentiates records from
    the < 4.1.x storage format.
 3. The third field is the page number.
 4. The fourth field contains the type info, where we have also added 2 bytes to
@@ -55,7 +55,14 @@ insert buffer tree, and that is in the system tablespace of InnoDB.
    can use in the binary search on the index page in the ibuf merge phase.
 5. The rest of the fields contain the fields of the actual index record.
 
-*/
+In versions >= 5.0.3:
+
+The first byte of the fourth field is an additional marker (0) if the record
+is in the compact format.  The presence of this marker can be detected by
+looking at the length of the field modulo DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE.
+
+The high-order bit of the character set field in the type info is the
+"nullable" flag for the field. */
 
 
 /*	PREVENTING DEADLOCKS IN THE INSERT BUFFER SYSTEM
@@ -525,8 +532,8 @@ ibuf_data_init_for_space(
 	ibuf_exit();
 
 	sprintf(buf, "SYS_IBUF_TABLE_%lu", (ulong) space);
-	
-	table = dict_mem_table_create(buf, space, 2);
+	/* use old-style record format for the insert buffer */
+	table = dict_mem_table_create(buf, space, 2, FALSE);
 
 	dict_mem_table_add_col(table, "PAGE_NO", DATA_BINARY, 0, 0, 0);
 	dict_mem_table_add_col(table, "TYPES", DATA_BINARY, 0, 0, 0);
@@ -541,11 +548,9 @@ ibuf_data_init_for_space(
 	dict_mem_index_add_field(index, "PAGE_NO", 0, 0);
 	dict_mem_index_add_field(index, "TYPES", 0, 0);
 
-	index->page_no = FSP_IBUF_TREE_ROOT_PAGE_NO;
-	
 	index->id = ut_dulint_add(DICT_IBUF_ID_MIN, space);
 
-	dict_index_add_to_cache(table, index);
+	dict_index_add_to_cache(table, index, FSP_IBUF_TREE_ROOT_PAGE_NO);
 
 	data->index = dict_table_get_first_index(table);
 
@@ -945,14 +950,11 @@ ibool
 ibuf_fixed_addr_page(
 /*=================*/
 			/* out: TRUE if a fixed address ibuf i/o page */
+	ulint	space,	/* in: space id */
 	ulint	page_no)/* in: page number */
 {
-	if ((ibuf_bitmap_page(page_no))
-				|| (page_no == IBUF_TREE_ROOT_PAGE_NO)) {
-		return(TRUE);
-	}
-
-	return(FALSE);
+	return((space == 0 && page_no == IBUF_TREE_ROOT_PAGE_NO)
+			|| ibuf_bitmap_page(page_no));
 }
 
 /***************************************************************************
@@ -976,7 +978,7 @@ ibuf_page(
 		return(FALSE);
 	}
 
-	if (ibuf_fixed_addr_page(page_no)) {
+	if (ibuf_fixed_addr_page(space, page_no)) {
 
 		return(TRUE);
 	}
@@ -1024,7 +1026,7 @@ ibuf_page_low(
 		return(FALSE);
 	}
 #endif	
-	if (ibuf_fixed_addr_page(page_no)) {
+	if (ibuf_fixed_addr_page(space, page_no)) {
 
 		return(TRUE);
 	}
@@ -1049,20 +1051,20 @@ ibuf_rec_get_page_no(
 	ulint	len;
 
 	ut_ad(ibuf_inside());
-	ut_ad(rec_get_n_fields(rec) > 2);
+	ut_ad(rec_get_n_fields_old(rec) > 2);
 
-	field = rec_get_nth_field(rec, 1, &len);
+	field = rec_get_nth_field_old(rec, 1, &len);
 
 	if (len == 1) {
 		/* This is of the >= 4.1.x record format */
 		ut_a(trx_sys_multiple_tablespace_format);
 
-		field = rec_get_nth_field(rec, 2, &len);
+		field = rec_get_nth_field_old(rec, 2, &len);
 	} else {
 		ut_a(trx_doublewrite_must_reset_space_ids);
 		ut_a(!trx_sys_multiple_tablespace_format);
 
-	        field = rec_get_nth_field(rec, 0, &len);
+		field = rec_get_nth_field_old(rec, 0, &len);
 	}
 
 	ut_a(len == 4);
@@ -1084,15 +1086,15 @@ ibuf_rec_get_space(
 	ulint	len;
 
 	ut_ad(ibuf_inside());
-	ut_ad(rec_get_n_fields(rec) > 2);
+	ut_ad(rec_get_n_fields_old(rec) > 2);
 
-	field = rec_get_nth_field(rec, 1, &len);
+	field = rec_get_nth_field_old(rec, 1, &len);
 
 	if (len == 1) {
 		/* This is of the >= 4.1.x record format */
 
 		ut_a(trx_sys_multiple_tablespace_format);
-		field = rec_get_nth_field(rec, 0, &len);
+		field = rec_get_nth_field_old(rec, 0, &len);
 		ut_a(len == 4);
 
 		return(mach_read_from_4(field));
@@ -1104,6 +1106,162 @@ ibuf_rec_get_space(
 	return(0);
 }
 
+/************************************************************************
+Creates a dummy index for inserting a record to a non-clustered index.
+*/
+static
+dict_index_t*
+ibuf_dummy_index_create(
+/*====================*/
+				/* out: dummy index */
+	ulint		n,	/* in: number of fields */
+	ibool		comp)	/* in: TRUE=use compact record format */
+{
+	dict_table_t*	table;
+	dict_index_t*	index;
+	table = dict_mem_table_create("IBUF_DUMMY",
+			DICT_HDR_SPACE, n, comp);
+	index = dict_mem_index_create("IBUF_DUMMY", "IBUF_DUMMY",
+			DICT_HDR_SPACE, 0, n);
+	index->table = table;
+	/* avoid ut_ad(index->cached) in dict_index_get_n_unique_in_tree */
+	index->cached = TRUE;
+	return(index);
+}
+/************************************************************************
+Add a column to the dummy index */
+static
+void
+ibuf_dummy_index_add_col(
+/*====================*/
+	dict_index_t*	index,	/* in: dummy index */
+	dtype_t*	type,	/* in: the data type of the column */
+	ulint		len)	/* in: length of the column */
+{
+	ulint	i	= index->table->n_def;
+	dict_mem_table_add_col(index->table, "DUMMY",
+		dtype_get_mtype(type),
+		dtype_get_prtype(type),
+		dtype_get_len(type),
+		dtype_get_prec(type));
+	dict_index_add_col(index,
+		dict_table_get_nth_col(index->table, i), 0, len);
+}
+/************************************************************************
+Deallocates a dummy index for inserting a record to a non-clustered index.
+*/
+static
+void
+ibuf_dummy_index_free(
+/*====================*/
+	dict_index_t*	index)	/* in: dummy index */
+{
+	dict_table_t*	table = index->table;
+
+	dict_mem_index_free(index);
+	dict_mem_table_free(table);
+}
+
+/*************************************************************************
+Builds the entry to insert into a non-clustered index when we have the
+corresponding record in an ibuf index. */
+static
+dtuple_t*
+ibuf_build_entry_from_ibuf_rec(
+/*===========================*/
+					/* out, own: entry to insert to
+					a non-clustered index; NOTE that
+					as we copy pointers to fields in
+					ibuf_rec, the caller must hold a
+					latch to the ibuf_rec page as long
+					as the entry is used! */
+	rec_t*		ibuf_rec,	/* in: record in an insert buffer */
+	mem_heap_t*	heap,		/* in: heap where built */
+	dict_index_t**	pindex)		/* out, own: dummy index that
+					describes the entry */
+{
+	dtuple_t*	tuple;
+	dfield_t*	field;
+	ulint		n_fields;
+	byte*		types;
+	const byte*	data;
+	ulint		len;
+	ulint		i;
+	dict_index_t*	index;
+
+	data = rec_get_nth_field_old(ibuf_rec, 1, &len);
+
+	if (len > 1) {
+		/* This a < 4.1.x format record */
+
+		ut_a(trx_doublewrite_must_reset_space_ids);
+		ut_a(!trx_sys_multiple_tablespace_format);
+
+		n_fields = rec_get_n_fields_old(ibuf_rec) - 2;
+		tuple = dtuple_create(heap, n_fields);
+		types = rec_get_nth_field_old(ibuf_rec, 1, &len);
+
+		ut_a(len == n_fields * DATA_ORDER_NULL_TYPE_BUF_SIZE);
+
+		for (i = 0; i < n_fields; i++) {
+			field = dtuple_get_nth_field(tuple, i);
+
+			data = rec_get_nth_field_old(ibuf_rec, i + 2, &len);
+
+			dfield_set_data(field, data, len);
+
+			dtype_read_for_order_and_null_size(
+				dfield_get_type(field),
+				types + i * DATA_ORDER_NULL_TYPE_BUF_SIZE);
+		}
+
+		*pindex = ibuf_dummy_index_create(n_fields, FALSE);
+		return(tuple);
+	}
+
+	/* This a >= 4.1.x format record */
+
+	ut_a(trx_sys_multiple_tablespace_format);
+	ut_a(*data == 0);
+	ut_a(rec_get_n_fields_old(ibuf_rec) > 4);
+
+	n_fields = rec_get_n_fields_old(ibuf_rec) - 4;
+
+	tuple = dtuple_create(heap, n_fields);
+
+	types = rec_get_nth_field_old(ibuf_rec, 3, &len);
+
+	ut_a(len % DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE <= 1);
+	index = ibuf_dummy_index_create(n_fields,
+				len % DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE);
+
+	if (len % DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE) {
+		/* compact record format */
+		len--;
+		ut_a(*types == 0);
+		types++;
+	}
+
+	ut_a(len == n_fields * DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE);
+
+	for (i = 0; i < n_fields; i++) {
+		field = dtuple_get_nth_field(tuple, i);
+
+		data = rec_get_nth_field_old(ibuf_rec, i + 4, &len);
+
+		dfield_set_data(field, data, len);
+
+		dtype_new_read_for_order_and_null_size(
+			dfield_get_type(field),
+			types + i * DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE);
+
+		ibuf_dummy_index_add_col(index, dfield_get_type(field), len);
+	}
+
+	*pindex = index;
+	return(tuple);
+}
+
 /************************************************************************
 Returns the space taken by a stored non-clustered index entry if converted to
 an index record. */
@@ -1125,43 +1283,60 @@ ibuf_rec_get_volume(
 	ulint	i;
 
 	ut_ad(ibuf_inside());
-	ut_ad(rec_get_n_fields(ibuf_rec) > 2);
-	
-	data = rec_get_nth_field(ibuf_rec, 1, &len);
+	ut_ad(rec_get_n_fields_old(ibuf_rec) > 2);
+
+	data = rec_get_nth_field_old(ibuf_rec, 1, &len);
 
 	if (len > 1) {
-	        /* < 4.1.x format record */
+		/* < 4.1.x format record */
 
 		ut_a(trx_doublewrite_must_reset_space_ids);
 		ut_a(!trx_sys_multiple_tablespace_format);
 
-		n_fields = rec_get_n_fields(ibuf_rec) - 2;
+		n_fields = rec_get_n_fields_old(ibuf_rec) - 2;
 
-		types = rec_get_nth_field(ibuf_rec, 1, &len);
+		types = rec_get_nth_field_old(ibuf_rec, 1, &len);
 
 		ut_ad(len == n_fields * DATA_ORDER_NULL_TYPE_BUF_SIZE);
 	} else {
-	        /* >= 4.1.x format record */
+		/* >= 4.1.x format record */
 
 		ut_a(trx_sys_multiple_tablespace_format);
+		ut_a(*data == 0);
+
+		types = rec_get_nth_field_old(ibuf_rec, 3, &len);
+
+		ut_a(len % DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE <= 1);
+		if (len % DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE) {
+			/* compact record format */
+			ulint		volume;
+			dict_index_t*	dummy_index;
+			mem_heap_t*	heap = mem_heap_create(500);
+			dtuple_t*	entry =
+				ibuf_build_entry_from_ibuf_rec(
+					ibuf_rec, heap, &dummy_index);
+			volume = rec_get_converted_size(dummy_index, entry);
+			ibuf_dummy_index_free(dummy_index);
+			mem_heap_free(heap);
+			return(volume + page_dir_calc_reserved_space(1));
+		}
+
+		n_fields = rec_get_n_fields_old(ibuf_rec) - 4;
+
 		new_format = TRUE;
-
-		n_fields = rec_get_n_fields(ibuf_rec) - 4;
-
-		types = rec_get_nth_field(ibuf_rec, 3, &len);
 	}
 
 	for (i = 0; i < n_fields; i++) {
 		if (new_format) {
-		        data = rec_get_nth_field(ibuf_rec, i + 4, &len);
+			data = rec_get_nth_field_old(ibuf_rec, i + 4, &len);
 
 			dtype_new_read_for_order_and_null_size(&dtype,
-			       types + i * DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE);
+				types + i * DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE);
 		} else {
-		        data = rec_get_nth_field(ibuf_rec, i + 2, &len);
+			data = rec_get_nth_field_old(ibuf_rec, i + 2, &len);
 
 			dtype_read_for_order_and_null_size(&dtype,
-				   types + i * DATA_ORDER_NULL_TYPE_BUF_SIZE);
+				types + i * DATA_ORDER_NULL_TYPE_BUF_SIZE);
 		}
 
 		if (len == UNIV_SQL_NULL) {
@@ -1186,6 +1361,7 @@ ibuf_entry_build(
 				index tree; NOTE that the original entry
 				must be kept because we copy pointers to its
 				fields */
+	dict_index_t*	index,	/* in: non-clustered index */
 	dtuple_t*	entry,	/* in: entry for a non-clustered index */
 	ulint		space,	/* in: space id */
 	ulint		page_no,/* in: index page number where entry should
@@ -1202,11 +1378,14 @@ ibuf_entry_build(
 	
 	/* Starting from 4.1.x, we have to build a tuple whose
 	(1) first field is the space id,
-	(2) the second field a single marker byte to tell that this
+	(2) the second field a single marker byte (0) to tell that this
 	is a new format record,
 	(3) the third contains the page number, and
 	(4) the fourth contains the relevent type information of each data
-	field,
+	field; the length of this field % DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE is
+	 (a) 0 for b-trees in the old format, and
+	 (b) 1 for b-trees in the compact format, the first byte of the field
+	 being the marker (0);
 	(5) and the rest of the fields are copied from entry. All fields
 	in the tuple are ordered like the type binary in our insert buffer
 	tree. */
@@ -1247,10 +1426,15 @@ ibuf_entry_build(
 
 	dfield_set_data(field, buf, 4);
 
+	ut_ad(index->table->comp <= 1);
 	/* Store the type info in buf2, and add the fields from entry to
 	tuple */
 	buf2 = mem_heap_alloc(heap, n_fields
-					* DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE);
+					* DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE
+					+ index->table->comp);
+	if (index->table->comp) {
+		*buf2++ = 0; /* write the compact format indicator */
+	}
 	for (i = 0; i < n_fields; i++) {
 		/* We add 4 below because we have the 4 extra fields at the
 		start of an ibuf record */
@@ -1261,15 +1445,22 @@ ibuf_entry_build(
 
 		dtype_new_store_for_order_and_null_size(
 				buf2 + i * DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE,
-				dfield_get_type(entry_field));
+				dfield_get_type(entry_field),
+				dict_index_get_nth_field(index, i)
+				->prefix_len);
 	}
 
 	/* Store the type info in buf2 to field 3 of tuple */
 
 	field = dtuple_get_nth_field(tuple, 3);
 
+	if (index->table->comp) {
+		buf2--;
+	}
+
 	dfield_set_data(field, buf2, n_fields
-					* DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE);
+					* DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE
+					+ index->table->comp);
 	/* Set all the types in the new tuple binary */
 
 	dtuple_set_types_binary(tuple, n_fields + 4);
@@ -1277,88 +1468,6 @@ ibuf_entry_build(
 	return(tuple);
 }	
 
-/*************************************************************************
-Builds the entry to insert into a non-clustered index when we have the
-corresponding record in an ibuf index. */
-static
-dtuple_t*
-ibuf_build_entry_from_ibuf_rec(
-/*===========================*/
-					/* out, own: entry to insert to
-					a non-clustered index; NOTE that
-					as we copy pointers to fields in
-					ibuf_rec, the caller must hold a
-					latch to the ibuf_rec page as long
-					as the entry is used! */
-	rec_t*		ibuf_rec,	/* in: record in an insert buffer */
-	mem_heap_t*	heap)		/* in: heap where built */
-{
-	dtuple_t*	tuple;
-	dfield_t*	field;
-	ulint		n_fields;
-	byte*		types;
-	byte*		data;
-	ulint		len;
-	ulint		i;
-	
-	data = rec_get_nth_field(ibuf_rec, 1, &len);
-
-	if (len > 1) {
-	        /* This a < 4.1.x format record */
-
-		ut_a(trx_doublewrite_must_reset_space_ids);
-		ut_a(!trx_sys_multiple_tablespace_format);
-
-		n_fields = rec_get_n_fields(ibuf_rec) - 2;
-		tuple = dtuple_create(heap, n_fields);
-		types = rec_get_nth_field(ibuf_rec, 1, &len);
-
-		ut_a(len == n_fields * DATA_ORDER_NULL_TYPE_BUF_SIZE);
-
-		for (i = 0; i < n_fields; i++) {
-		        field = dtuple_get_nth_field(tuple, i);
-
-			data = rec_get_nth_field(ibuf_rec, i + 2, &len);
-
-			dfield_set_data(field, data, len);
-
-			dtype_read_for_order_and_null_size(
-				   dfield_get_type(field),
-				   types + i * DATA_ORDER_NULL_TYPE_BUF_SIZE);
-		}
-
-		return(tuple);
-	}
-
-	/* This a >= 4.1.x format record */
-
-	ut_a(trx_sys_multiple_tablespace_format);
-
-	ut_a(rec_get_n_fields(ibuf_rec) > 4);
-
-	n_fields = rec_get_n_fields(ibuf_rec) - 4;
-
-	tuple = dtuple_create(heap, n_fields);
-
-	types = rec_get_nth_field(ibuf_rec, 3, &len);
-
-	ut_a(len == n_fields * DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE);
-
-	for (i = 0; i < n_fields; i++) {
-	        field = dtuple_get_nth_field(tuple, i);
-
-		data = rec_get_nth_field(ibuf_rec, i + 4, &len);
-
-		dfield_set_data(field, data, len);
-
-		dtype_new_read_for_order_and_null_size(
-			dfield_get_type(field),
-			types + i * DATA_NEW_ORDER_NULL_TYPE_BUF_SIZE);
-	}
-
-	return(tuple);
-}
-
 /*************************************************************************
 Builds a search tuple used to search buffered inserts for an index page.
 This is for < 4.1.x format records */
@@ -1779,7 +1888,7 @@ ibuf_get_merge_page_nos(
 				contract the tree, FALSE if this is called
 				when a single page becomes full and we look
 				if it pays to read also nearby pages */
-	rec_t*		first_rec,/* in: record from which we read up and down
+	rec_t*		rec,	/* in: record from which we read up and down
 				in the chain of records */
 	ulint*		space_ids,/* in/out: space id's of the pages */
 	ib_longlong*	space_versions,/* in/out: tablespace version
@@ -1797,47 +1906,42 @@ ibuf_get_merge_page_nos(
 	ulint	first_space_id;
 	ulint	rec_page_no;
 	ulint	rec_space_id;
-	rec_t*	rec;
 	ulint	sum_volumes;
 	ulint	volume_for_page;
 	ulint	rec_volume;
 	ulint	limit;
-	page_t*	page;
 	ulint	n_pages;
 
 	*n_stored = 0;
 
 	limit = ut_min(IBUF_MAX_N_PAGES_MERGED, buf_pool->curr_size / 4);
 
-	page = buf_frame_align(first_rec);
-	
-	if (first_rec == page_get_supremum_rec(page)) {
+	if (page_rec_is_supremum(rec)) {
 
-		first_rec = page_rec_get_prev(first_rec);
+		rec = page_rec_get_prev(rec);
 	}
 
-	if (first_rec == page_get_infimum_rec(page)) {
+	if (page_rec_is_infimum(rec)) {
 
-		first_rec = page_rec_get_next(first_rec);
+		rec = page_rec_get_next(rec);
 	}
 
-	if (first_rec == page_get_supremum_rec(page)) {
+	if (page_rec_is_supremum(rec)) {
 
 		return(0);
 	}
 
-	rec = first_rec;
-	first_page_no = ibuf_rec_get_page_no(first_rec);
-	first_space_id = ibuf_rec_get_space(first_rec);
+	first_page_no = ibuf_rec_get_page_no(rec);
+	first_space_id = ibuf_rec_get_space(rec);
 	n_pages = 0;
 	prev_page_no = 0;
 	prev_space_id = 0;
 	
-	/* Go backwards from the first_rec until we reach the border of the
+	/* Go backwards from the first rec until we reach the border of the
 	'merge area', or the page start or the limit of storeable pages is
 	reached */
 
-	while ((rec != page_get_infimum_rec(page)) && (n_pages < limit)) {
+	while (!page_rec_is_infimum(rec) && UNIV_LIKELY(n_pages < limit)) {
 
 		rec_page_no = ibuf_rec_get_page_no(rec);
 		rec_space_id = ibuf_rec_get_space(rec);
@@ -1872,7 +1976,7 @@ ibuf_get_merge_page_nos(
 	volume_for_page = 0;
 	
 	while (*n_stored < limit) {
-		if (rec == page_get_supremum_rec(page)) {
+		if (page_rec_is_supremum(rec)) {
 			/* When no more records available, mark this with
 			another 'impossible' pair of space id, page no */
 			rec_page_no = 1;
@@ -2047,8 +2151,7 @@ loop:
 	mutex_exit(&ibuf_mutex);
 
 	sum_sizes = ibuf_get_merge_page_nos(TRUE, btr_pcur_get_rec(&pcur),
-					space_ids, space_versions, page_nos,
-					&n_stored);
+			space_ids, space_versions, page_nos, &n_stored);
 #ifdef UNIV_IBUF_DEBUG
 	/* fprintf(stderr, "Ibuf contract sync %lu pages %lu volume %lu\n",
 		sync, n_stored, sum_sizes); */
@@ -2202,12 +2305,12 @@ ibuf_get_volume_buffered(
 
 	page = buf_frame_align(rec);
 
-	if (rec == page_get_supremum_rec(page)) {
+	if (page_rec_is_supremum(rec)) {
 		rec = page_rec_get_prev(rec);
 	}
 
 	for (;;) {
-		if (rec == page_get_infimum_rec(page)) {
+		if (page_rec_is_infimum(rec)) {
 
 			break;
 		}
@@ -2242,7 +2345,7 @@ ibuf_get_volume_buffered(
 	rec = page_rec_get_prev(rec);
 	
 	for (;;) {
-		if (rec == page_get_infimum_rec(prev_page)) {
+		if (page_rec_is_infimum(rec)) {
 
 			/* We cannot go to yet a previous page, because we
 			do not have the x-latch on it, and cannot acquire one
@@ -2265,12 +2368,12 @@ ibuf_get_volume_buffered(
 count_later:
 	rec = btr_pcur_get_rec(pcur);
 
-	if (rec != page_get_supremum_rec(page)) {
+	if (!page_rec_is_supremum(rec)) {
 		rec = page_rec_get_next(rec);
 	}
 
 	for (;;) {
-		if (rec == page_get_supremum_rec(page)) {
+		if (page_rec_is_supremum(rec)) {
 
 			break;
 		}
@@ -2305,7 +2408,7 @@ count_later:
 	rec = page_rec_get_next(rec);
 
 	for (;;) {
-		if (rec == page_get_supremum_rec(next_page)) {
+		if (page_rec_is_supremum(rec)) {
 
 			/* We give up */
 		
@@ -2344,6 +2447,7 @@ ibuf_update_max_tablespace_id(void)
 	ibuf_data = fil_space_get_ibuf_data(0);
 
 	ibuf_index = ibuf_data->index;
+	ut_a(!ibuf_index->table->comp);
 
 	ibuf_enter();
 
@@ -2360,7 +2464,7 @@ ibuf_update_max_tablespace_id(void)
 	} else {
 		rec = btr_pcur_get_rec(&pcur);
 
-		field = rec_get_nth_field(rec, 0, &len);
+		field = rec_get_nth_field_old(rec, 0, &len);
 
 		ut_a(len == 4);
 		
@@ -2479,7 +2583,7 @@ ibuf_insert_low(
 		ibuf_enter();
 	}
 
-	entry_size = rec_get_converted_size(entry);
+	entry_size = rec_get_converted_size(index, entry);
 
 	heap = mem_heap_create(512);
 
@@ -2487,7 +2591,7 @@ ibuf_insert_low(
 	the first fields and the type information for other fields, and which
 	will be inserted to the insert buffer. */
 
-	ibuf_entry = ibuf_entry_build(entry, space, page_no, heap);
+	ibuf_entry = ibuf_entry_build(index, entry, space, page_no, heap);
 
 	/* Open a cursor to the insert buffer tree to calculate if we can add
 	the new entry to it without exceeding the free space limit for the
@@ -2532,8 +2636,8 @@ ibuf_insert_low(
 		do_merge = TRUE; 
 
 		ibuf_get_merge_page_nos(FALSE, btr_pcur_get_rec(&pcur),
-					space_ids, space_versions, page_nos,
-					&n_stored);
+					space_ids, space_versions,
+					page_nos, &n_stored);
 		goto function_exit;
  	}
 
@@ -2656,8 +2760,8 @@ ibuf_insert(
 
 	ut_a(!(index->type & DICT_CLUSTERED));
 	
-	if (rec_get_converted_size(entry)
-				>= page_get_free_space_of_empty() / 2) {
+	if (rec_get_converted_size(index, entry)
+		>= page_get_free_space_of_empty(index->table->comp) / 2) {
 		return(FALSE);
 	}
 	
@@ -2692,6 +2796,7 @@ ibuf_insert_to_index_page(
 	dtuple_t*	entry,	/* in: buffered entry to insert */
 	page_t*		page,	/* in: index page where the buffered entry
 				should be placed */
+	dict_index_t*	index,	/* in: record descriptor */
 	mtr_t*		mtr)	/* in: mtr */
 {
 	page_cur_t	page_cur;
@@ -2703,13 +2808,21 @@ ibuf_insert_to_index_page(
 	ut_ad(ibuf_inside());
 	ut_ad(dtuple_check_typed(entry));
 
-	if (rec_get_n_fields(page_rec_get_next(page_get_infimum_rec(page)))
-	    != dtuple_get_n_fields(entry)) {
-
-		fprintf(stderr,
+	if (UNIV_UNLIKELY(index->table->comp != (ibool)!!page_is_comp(page))) {
+		fputs(
 "InnoDB: Trying to insert a record from the insert buffer to an index page\n"
-"InnoDB: but the number of fields does not match!\n");
+"InnoDB: but the 'compact' flag does not match!\n", stderr);
+		goto dump;
+	}
 
+	rec = page_rec_get_next(page_get_infimum_rec(page));
+
+	if (UNIV_UNLIKELY(rec_get_n_fields(rec, index)
+			!= dtuple_get_n_fields(entry))) {
+		fputs(
+"InnoDB: Trying to insert a record from the insert buffer to an index page\n"
+"InnoDB: but the number of fields does not match!\n", stderr);
+	dump:
 		buf_page_print(page);
 
 	        dtuple_print(stderr, entry);
@@ -2723,31 +2836,34 @@ ibuf_insert_to_index_page(
 		return;
 	}
 
-	low_match = page_cur_search(page, entry, PAGE_CUR_LE, &page_cur);
+	low_match = page_cur_search(page, index, entry,
+						PAGE_CUR_LE, &page_cur);
 	
 	if (low_match == dtuple_get_n_fields(entry)) {
 		rec = page_cur_get_rec(&page_cur);
 		
 		btr_cur_del_unmark_for_ibuf(rec, mtr);
 	} else {
-		rec = page_cur_tuple_insert(&page_cur, entry, mtr);
+		rec = page_cur_tuple_insert(&page_cur, entry, index, mtr);
 		
 		if (rec == NULL) {
 			/* If the record did not fit, reorganize */
 
-			btr_page_reorganize(page, mtr);
+			btr_page_reorganize(page, index, mtr);
 
-			page_cur_search(page, entry, PAGE_CUR_LE, &page_cur);
+			page_cur_search(page, index, entry,
+						PAGE_CUR_LE, &page_cur);
 
 			/* This time the record must fit */
-			if (!page_cur_tuple_insert(&page_cur, entry, mtr)) {
+			if (UNIV_UNLIKELY(!page_cur_tuple_insert(
+					&page_cur, entry, index, mtr))) {
 
 				ut_print_timestamp(stderr);
 
 				fprintf(stderr,
 "InnoDB: Error: Insert buffer insert fails; page free %lu, dtuple size %lu\n",
 				(ulong) page_get_max_insert_size(page, 1),
-				(ulong) rec_get_converted_size(entry));
+				(ulong) rec_get_converted_size(index, entry));
 				fputs("InnoDB: Cannot insert index record ",
 					stderr);
 				dtuple_print(stderr, entry);
@@ -2836,17 +2952,20 @@ ibuf_delete_rec(
 		"InnoDB: ibuf record inserted to page %lu\n", (ulong) page_no);
 		fflush(stderr);
 
-		rec_print(stderr, btr_pcur_get_rec(pcur));
-		rec_print(stderr, pcur->old_rec);
+		rec_print_old(stderr, btr_pcur_get_rec(pcur));
+		rec_print_old(stderr, pcur->old_rec);
 		dtuple_print(stderr, search_tuple);
 
-		rec_print(stderr, page_rec_get_next(btr_pcur_get_rec(pcur)));
+		rec_print_old(stderr,
+				page_rec_get_next(btr_pcur_get_rec(pcur)));
 		fflush(stderr);
 
 		btr_pcur_commit_specify_mtr(pcur, mtr);
 
 		fputs("InnoDB: Validating insert buffer tree:\n", stderr);
-		ut_a(btr_validate_tree(ibuf_data->index->tree));
+		if (!btr_validate_tree(ibuf_data->index->tree, NULL)) {
+			ut_error;
+		}
 
 		fprintf(stderr, "InnoDB: ibuf tree ok\n");
 		fflush(stderr);
@@ -2931,7 +3050,7 @@ ibuf_merge_or_delete_for_page(
 		return;
 	}
 #endif	
-	if (ibuf_fixed_addr_page(page_no) || fsp_descr_page(page_no)
+	if (ibuf_fixed_addr_page(space, page_no) || fsp_descr_page(page_no)
 					|| trx_sys_hdr_page(space, page_no)) {
 		return;
 	}
@@ -3075,7 +3194,7 @@ loop:
 
 		if (corruption_noticed) {
 			fputs("InnoDB: Discarding record\n ", stderr);
-			rec_print(stderr, ibuf_rec);
+			rec_print_old(stderr, ibuf_rec);
 			fputs("\n from the insert buffer!\n\n", stderr);
 	   	} else if (page) {
 			/* Now we have at pcur a record which should be
@@ -3083,19 +3202,22 @@ loop:
 			copies pointers to fields in ibuf_rec, and we must
 			keep the latch to the ibuf_rec page until the
 			insertion is finished! */
-
-			dulint	max_trx_id = page_get_max_trx_id(
+			dict_index_t*	dummy_index;
+			dulint		max_trx_id = page_get_max_trx_id(
 						buf_frame_align(ibuf_rec));
 			page_update_max_trx_id(page, max_trx_id);
 			
-			entry = ibuf_build_entry_from_ibuf_rec(ibuf_rec, heap);
+			entry = ibuf_build_entry_from_ibuf_rec(ibuf_rec,
+							heap, &dummy_index);
 #ifdef UNIV_IBUF_DEBUG
-			volume += rec_get_converted_size(entry)
+			volume += rec_get_converted_size(dummy_index, entry)
  					+ page_dir_calc_reserved_space(1);
 			ut_a(volume <= 4 * UNIV_PAGE_SIZE
 					/ IBUF_PAGE_SIZE_PER_FREE_SPACE);
 #endif
-			ibuf_insert_to_index_page(entry, page, &mtr);
+			ibuf_insert_to_index_page(entry, page,
+						dummy_index, &mtr);
+			ibuf_dummy_index_free(dummy_index);
 		}
 
 		n_inserts++;
@@ -3378,21 +3500,9 @@ ibuf_print(
 	data = UT_LIST_GET_FIRST(ibuf->data_list);
 
 	while (data) {
-	fprintf(file,
-  	"Ibuf for space %lu: size %lu, free list len %lu, seg size %lu,",
-			       (ulong) data->space, (ulong) data->size,
-			       (ulong) data->free_list_len,
-			       (ulong) data->seg_size);
-
-		if (data->empty) {
-			fputs(" is empty\n", file);
-		} else {
-			fputs(" is not empty\n", file);
-		}
 		fprintf(file,
-	"Ibuf for space %lu: size %lu, free list len %lu, seg size %lu,\n"
-			"%lu inserts, %lu merged recs, %lu merges\n",
-                               (ulong) data->space,
+	"Ibuf: size %lu, free list len %lu, seg size %lu,\n"
+	"%lu inserts, %lu merged recs, %lu merges\n",
                                (ulong) data->size,
                                (ulong) data->free_list_len,
 			       (ulong) data->seg_size,
