@@ -79,7 +79,8 @@ void embedded_get_error(MYSQL *mysql, MYSQL_DATA *data)
 static my_bool
 emb_advanced_command(MYSQL *mysql, enum enum_server_command command,
 		     const char *header, ulong header_length,
-		     const char *arg, ulong arg_length, my_bool skip_check)
+		     const char *arg, ulong arg_length, my_bool skip_check,
+                     MYSQL_STMT *stmt)
 {
   my_bool result= 1;
   THD *thd=(THD *) mysql->thd;
@@ -99,6 +100,7 @@ emb_advanced_command(MYSQL *mysql, enum enum_server_command command,
   mysql->affected_rows= ~(my_ulonglong) 0;
   mysql->field_count= 0;
   net->last_errno= 0;
+  thd->current_stmt= stmt;
 
   thd->store_globals();				// Fix if more than one connect
   /* 
@@ -196,6 +198,7 @@ static my_bool emb_read_prepare_result(MYSQL *mysql, MYSQL_STMT *stmt)
   stmt->stmt_id= thd->client_stmt_id;
   stmt->param_count= thd->client_param_count;
   stmt->field_count= 0;
+  mysql->warning_count= thd->total_warn_count;
 
   if (thd->first_data)
   {
@@ -276,26 +279,27 @@ static int emb_stmt_execute(MYSQL_STMT *stmt)
 {
   DBUG_ENTER("emb_stmt_execute");
   char header[5];
-  MYSQL_DATA *res;
   THD *thd;
+  my_bool res;
 
   int4store(header, stmt->stmt_id);
   header[4]= stmt->flags;
   thd= (THD*)stmt->mysql->thd;
   thd->client_param_count= stmt->param_count;
   thd->client_params= stmt->params;
-  if (emb_advanced_command(stmt->mysql, COM_STMT_EXECUTE,0,0,
-                           header, sizeof(header), 1) ||
-      emb_read_query_result(stmt->mysql))
+
+  res= test(emb_advanced_command(stmt->mysql, COM_STMT_EXECUTE, 0, 0,
+                                 header, sizeof(header), 1, stmt) ||
+            emb_read_query_result(stmt->mysql));
+  stmt->affected_rows= stmt->mysql->affected_rows;
+  stmt->insert_id= stmt->mysql->insert_id;
+  stmt->server_status= stmt->mysql->server_status;
+  if (res)
   {
     NET *net= &stmt->mysql->net;
     set_stmt_errmsg(stmt, net->last_error, net->last_errno, net->sqlstate);
     DBUG_RETURN(1);
   }
-  stmt->affected_rows= stmt->mysql->affected_rows;
-  stmt->insert_id= stmt->mysql->insert_id;
-  stmt->server_status= stmt->mysql->server_status;
-
   DBUG_RETURN(0);
 }
 
@@ -303,7 +307,11 @@ int emb_read_binary_rows(MYSQL_STMT *stmt)
 {
   MYSQL_DATA *data;
   if (!(data= emb_read_rows(stmt->mysql, 0, 0)))
+  {
+    set_stmt_errmsg(stmt, stmt->mysql->net.last_error,
+                    stmt->mysql->net.last_errno, stmt->mysql->net.sqlstate);
     return 1;
+  }
   stmt->result= *data;
   my_free((char *) data, MYF(0));
   set_stmt_errmsg(stmt, stmt->mysql->net.last_error,
@@ -385,7 +393,6 @@ static MYSQL_RES * emb_store_result(MYSQL *mysql)
   return mysql_store_result(mysql);
 }
 
-
 int emb_read_change_user_result(MYSQL *mysql, 
 				char *buff __attribute__((unused)),
 				const char *passwd __attribute__((unused)))
@@ -549,7 +556,7 @@ void end_embedded_server()
 }
 
 
-void init_embedded_mysql(MYSQL *mysql, int client_flag, char *db)
+void init_embedded_mysql(MYSQL *mysql, int client_flag)
 {
   THD *thd = (THD *)mysql->thd;
   thd->mysql= mysql;
@@ -557,7 +564,7 @@ void init_embedded_mysql(MYSQL *mysql, int client_flag, char *db)
   init_alloc_root(&mysql->field_alloc, 8192, 0);
 }
 
-void *create_embedded_thd(int client_flag, char *db)
+void *create_embedded_thd(int client_flag)
 {
   THD * thd= new THD;
   thd->thread_id= thread_id++;
@@ -583,9 +590,10 @@ void *create_embedded_thd(int client_flag, char *db)
   thd->set_time();
   thd->init_for_queries();
   thd->client_capabilities= client_flag;
+  thd->real_id= (pthread_t) thd;
 
-  thd->db= db;
-  thd->db_length= db ? strip_sp(db) : 0;
+  thd->db= NULL;
+  thd->db_length= 0;
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   thd->security_ctx->db_access= DB_ACLS;
   thd->security_ctx->master_access= ~NO_ACCESS;
@@ -604,7 +612,7 @@ err:
 
 
 #ifdef NO_EMBEDDED_ACCESS_CHECKS
-int check_embedded_connection(MYSQL *mysql)
+int check_embedded_connection(MYSQL *mysql, const char *db)
 {
   int result;
   THD *thd= (THD*)mysql->thd;
@@ -614,13 +622,13 @@ int check_embedded_connection(MYSQL *mysql)
   sctx->host_or_ip= sctx->host= (char*) my_localhost;
   strmake(sctx->priv_host, (char*) my_localhost,  MAX_HOSTNAME-1);
   sctx->priv_user= sctx->user= my_strdup(mysql->user, MYF(0));
-  result= check_user(thd, COM_CONNECT, NULL, 0, thd->db, true);
+  result= check_user(thd, COM_CONNECT, NULL, 0, db, true);
   emb_read_query_result(mysql);
   return result;
 }
 
 #else
-int check_embedded_connection(MYSQL *mysql)
+int check_embedded_connection(MYSQL *mysql, const char *db)
 {
   THD *thd= (THD*)mysql->thd;
   Security_context *sctx= thd->security_ctx;
@@ -657,7 +665,7 @@ int check_embedded_connection(MYSQL *mysql)
     passwd_len= 0;
 
   if((result= check_user(thd, COM_CONNECT, 
-			 scramble_buff, passwd_len, thd->db, true)))
+			 scramble_buff, passwd_len, db, true)))
      goto err;
 
   return 0;
@@ -768,6 +776,8 @@ MYSQL_DATA *THD::alloc_new_dataset()
 
 static void write_eof_packet(THD *thd)
 {
+  if (!thd->mysql)            // bootstrap file handling
+    return;
   /*
     The following test should never be true, but it's better to do it
     because if 'is_fatal_error' is set the server is not going to execute
@@ -889,10 +899,14 @@ bool Protocol::send_fields(List<Item> *list, uint flags)
     }
     else
     {
+      uint max_char_len;
       /* With conversion */
       client_field->charsetnr= thd_cs->number;
-      uint char_len= server_field.length / item->collation.collation->mbmaxlen;
-      client_field->length= char_len * thd_cs->mbmaxlen;
+      max_char_len= (server_field.type >= (int) MYSQL_TYPE_TINY_BLOB &&
+                     server_field.type <= (int) MYSQL_TYPE_BLOB) ?
+                     server_field.length / item->collation.collation->mbminlen :
+                     server_field.length / item->collation.collation->mbmaxlen;
+      client_field->length= max_char_len * thd_cs->mbmaxlen;
     }
     client_field->type=   server_field.type;
     client_field->flags= server_field.flags;
@@ -1023,6 +1037,9 @@ void Protocol_simple::prepare_for_resend()
   MYSQL_DATA *data= thd->cur_data;
   DBUG_ENTER("send_data");
 
+  if (!thd->mysql)            // bootstrap file handling
+    DBUG_VOID_RETURN;
+
   data->rows++;
   if (!(cur= (MYSQL_ROWS *)alloc_root(alloc, sizeof(MYSQL_ROWS)+(field_count + 1) * sizeof(char *))))
   {
