@@ -1,9 +1,8 @@
-/* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+/* Copyright (C) 2001-2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +19,6 @@
 
 #define FT_CORE
 #include "ftdefs.h"
-#include <queues.h>
 
 /* search with boolean queries */
 
@@ -54,54 +52,57 @@ static double _nwghts[11]=
  -3.796875000000000};
 static double *nwghts=_nwghts+5; /* nwghts[i] = -0.5*1.5**i */
 
-#define FTB_FLAG_TRUNC 1                  /* MUST be 1                  */
-#define FTB_FLAG_YES   2                  /*  no two from these three   */
-#define FTB_FLAG_NO    4                  /*   YES, NO, WONLY           */
-#define FTB_FLAG_WONLY 8                  /*  should be ever set both   */
+#define FTB_FLAG_TRUNC 1
+/* At most one of the following flags can be set */
+#define FTB_FLAG_YES   2
+#define FTB_FLAG_NO    4
+#define FTB_FLAG_WONLY 8
 
 typedef struct st_ftb_expr FTB_EXPR;
 struct st_ftb_expr
 {
   FTB_EXPR *up;
-  float     weight;
   uint      flags;
-  my_off_t  docid[2];
 /* ^^^^^^^^^^^^^^^^^^ FTB_{EXPR,WORD} common section */
+  my_off_t  docid[2];
+  float     weight;
   float     cur_weight;
-  byte     *quot, *qend;
-  int       yesses;               /* number of "yes" words matched */
-  int       nos;                  /* number of "no"  words matched */
-  int       ythresh;              /* number of "yes" words in expr */
-  int       yweaks;               /* number of "yes" words for scan only */
+  LIST     *phrase;               /* phrase words */
+  uint      yesses;               /* number of "yes" words matched */
+  uint      nos;                  /* number of "no"  words matched */
+  uint      ythresh;              /* number of "yes" words in expr */
+  uint      yweaks;               /* number of "yes" words for scan only */
 };
 
 typedef struct st_ftb_word
 {
-  FTB_EXPR *up;
-  float     weight;
-  uint      flags;
-  my_off_t  docid[2];
+  FTB_EXPR  *up;
+  uint       flags;
 /* ^^^^^^^^^^^^^^^^^^ FTB_{EXPR,WORD} common section */
-  uint      ndepth;
-  int       len;
-  /* ... docid cache can be added here. SerG */
-  byte      word[1];
+  my_off_t   docid[2];             /* for index search and for scan */
+  my_off_t   key_root;
+  MI_KEYDEF *keyinfo;
+  float      weight;
+  uint       ndepth;
+  uint       len;
+  uchar      off;
+  byte       word[1];
 } FTB_WORD;
 
 typedef struct st_ft_info
 {
   struct _ft_vft *please;
   MI_INFO   *info;
-  uint       keynr;
   CHARSET_INFO *charset;
-  enum { UNINITIALIZED, READY, INDEX_SEARCH, INDEX_DONE /*, SCAN*/ } state;
-  uint       with_scan;
-  my_off_t   lastpos;
   FTB_EXPR  *root;
-  QUEUE      queue;
-  TREE       no_dupes;
   FTB_WORD **list;
   MEM_ROOT   mem_root;
+  QUEUE      queue;
+  TREE       no_dupes;
+  my_off_t   lastpos;
+  uint       keynr;
+  uchar      with_scan;
+  enum { UNINITIALIZED, READY, INDEX_SEARCH, INDEX_DONE } state;
 } FTB;
 
 static int FTB_WORD_cmp(my_off_t *v, FTB_WORD *a, FTB_WORD *b)
@@ -122,29 +123,31 @@ static int FTB_WORD_cmp(my_off_t *v, FTB_WORD *a, FTB_WORD *b)
 static int FTB_WORD_cmp_list(CHARSET_INFO *cs, FTB_WORD **a, FTB_WORD **b)
 {
   /* ORDER BY word DESC, ndepth DESC */
-  int i=_mi_compare_text(cs, (uchar*) (*b)->word+1,(*b)->len-1,
-                             (uchar*) (*a)->word+1,(*a)->len-1,0);
+  int i= mi_compare_text(cs, (uchar*) (*b)->word+1,(*b)->len-1,
+                             (uchar*) (*a)->word+1,(*a)->len-1,0,0);
   if (!i)
     i=CMP_NUM((*b)->ndepth,(*a)->ndepth);
   return i;
 }
 
 static void _ftb_parse_query(FTB *ftb, byte **start, byte *end,
-                      FTB_EXPR *up, uint depth)
+                      FTB_EXPR *up, uint depth, byte *up_quot)
 {
   byte        res;
   FTB_PARAM   param;
   FT_WORD     w;
   FTB_WORD   *ftbw;
   FTB_EXPR   *ftbe;
+  FT_WORD    *phrase_word;
+  LIST       *phrase_list;
   uint  extra=HA_FT_WLEN+ftb->info->s->rec_reflength; /* just a shortcut */
 
   if (ftb->state != UNINITIALIZED)
     return;
 
   param.prev=' ';
-  param.quot=up->quot;
-  while ((res=ft_get_word(start,end,&w,&param)))
+  param.quot= up_quot;
+  while ((res=ft_get_word(ftb->charset,start,end,&w,&param)))
   {
     int   r=param.plusminus;
     float weight= (float) (param.pmsign ? nwghts : wghts)[(r>5)?5:((r<-5)?-5:r)];
@@ -153,9 +156,10 @@ static void _ftb_parse_query(FTB *ftb, byte **start, byte *end,
         ftbw=(FTB_WORD *)alloc_root(&ftb->mem_root,
                                     sizeof(FTB_WORD) +
                                     (param.trunc ? MI_MAX_KEY_BUFF :
-                                     w.len+extra));
+                                     w.len*ftb->charset->mbmaxlen+extra));
         ftbw->len=w.len+1;
         ftbw->flags=0;
+        ftbw->off=0;
         if (param.yesno>0) ftbw->flags|=FTB_FLAG_YES;
         if (param.yesno<0) ftbw->flags|=FTB_FLAG_NO;
         if (param.trunc)   ftbw->flags|=FTB_FLAG_TRUNC;
@@ -163,11 +167,20 @@ static void _ftb_parse_query(FTB *ftb, byte **start, byte *end,
         ftbw->up=up;
         ftbw->docid[0]=ftbw->docid[1]=HA_OFFSET_ERROR;
         ftbw->ndepth= (param.yesno<0) + depth;
+        ftbw->key_root=HA_OFFSET_ERROR;
         memcpy(ftbw->word+1, w.pos, w.len);
         ftbw->word[0]=w.len;
         if (param.yesno > 0) up->ythresh++;
         queue_insert(& ftb->queue, (byte *)ftbw);
         ftb->with_scan|=(param.trunc & FTB_FLAG_TRUNC);
+      case 4: /* not indexed word (stopword or too short/long) */
+        if (! up_quot) break;
+        phrase_word= (FT_WORD *)alloc_root(&ftb->mem_root, sizeof(FT_WORD));
+        phrase_list= (LIST *)alloc_root(&ftb->mem_root, sizeof(LIST));
+        phrase_word->pos= w.pos;
+        phrase_word->len= w.len;
+        phrase_list->data= (void *)phrase_word;
+        up->phrase= list_add(up->phrase, phrase_list);
         break;
       case 2: /* left bracket */
         ftbe=(FTB_EXPR *)alloc_root(&ftb->mem_root, sizeof(FTB_EXPR));
@@ -178,13 +191,14 @@ static void _ftb_parse_query(FTB *ftb, byte **start, byte *end,
         ftbe->up=up;
         ftbe->ythresh=ftbe->yweaks=0;
         ftbe->docid[0]=ftbe->docid[1]=HA_OFFSET_ERROR;
-        if ((ftbe->quot=param.quot)) ftb->with_scan|=2;
+        ftbe->phrase= NULL;
+        if (param.quot) ftb->with_scan|=2;
         if (param.yesno > 0) up->ythresh++;
-        _ftb_parse_query(ftb, start, end, ftbe, depth+1);
+        _ftb_parse_query(ftb, start, end, ftbe, depth+1, param.quot);
         param.quot=0;
         break;
       case 3: /* right bracket */
-        if (up->quot) up->qend=param.quot;
+        if (up_quot) up->phrase= list_reverse(up->phrase);
         return;
     }
   }
@@ -192,27 +206,126 @@ static void _ftb_parse_query(FTB *ftb, byte **start, byte *end,
 }
 
 static int _ftb_no_dupes_cmp(void* not_used __attribute__((unused)),
-			     const void *a,const void *b)
+                             const void *a,const void *b)
 {
   return CMP_NUM((*((my_off_t*)a)), (*((my_off_t*)b)));
 }
 
+/* returns 1 if the search was finished (must-word wasn't found) */
+static int _ft2_search(FTB *ftb, FTB_WORD *ftbw, my_bool init_search)
+{
+  int r;
+  int subkeys=1;
+  my_bool can_go_down;
+  MI_INFO *info=ftb->info;
+  uint off, extra=HA_FT_WLEN+info->s->base.rec_reflength;
+  byte *lastkey_buf=ftbw->word+ftbw->off;
+  LINT_INIT(off);
+
+  LINT_INIT(off);
+  if (ftbw->flags & FTB_FLAG_TRUNC)
+    lastkey_buf+=ftbw->len;
+
+  if (init_search)
+  {
+    ftbw->key_root=info->s->state.key_root[ftb->keynr];
+    ftbw->keyinfo=info->s->keyinfo+ftb->keynr;
+
+    r=_mi_search(info, ftbw->keyinfo, (uchar*) ftbw->word, ftbw->len,
+                 SEARCH_FIND | SEARCH_BIGGER, ftbw->key_root);
+  }
+  else
+  {
+    r=_mi_search(info, ftbw->keyinfo, (uchar*) lastkey_buf,
+                   USE_WHOLE_KEY, SEARCH_BIGGER, ftbw->key_root);
+  }
+
+  can_go_down=(!ftbw->off && (init_search || (ftbw->flags & FTB_FLAG_TRUNC)));
+  /* Skip rows inserted by concurrent insert */
+  while (!r)
+  {
+    if (can_go_down)
+    {
+      /* going down ? */
+      off=info->lastkey_length-extra;
+      subkeys=ft_sintXkorr(info->lastkey+off);
+    }
+    if (subkeys<0 || info->lastpos < info->state->data_file_length)
+      break;
+    r= _mi_search_next(info, ftbw->keyinfo, info->lastkey,
+                       info->lastkey_length,
+		       SEARCH_BIGGER, ftbw->key_root);
+  }
+
+  if (!r && !ftbw->off)
+  {
+    r= mi_compare_text(ftb->charset,
+                       info->lastkey+1,
+                       info->lastkey_length-extra-1,
+              (uchar*) ftbw->word+1,
+                       ftbw->len-1,
+             (my_bool) (ftbw->flags & FTB_FLAG_TRUNC),0);
+  }
+
+  if (r) /* not found */
+  {
+    if (!ftbw->off || !(ftbw->flags & FTB_FLAG_TRUNC))
+    {
+      ftbw->docid[0]=HA_OFFSET_ERROR;
+      if ((ftbw->flags & FTB_FLAG_YES) && ftbw->up->up==0)
+      {
+        /*
+          This word MUST BE present in every document returned,
+          so we can stop the search right now
+        */
+        ftb->state=INDEX_DONE;
+        return 1; /* search is done */
+      }
+      else
+        return 0;
+    }
+
+    /* going up to the first-level tree to continue search there */
+    _mi_dpointer(info, (uchar*) (lastkey_buf+HA_FT_WLEN), ftbw->key_root);
+    ftbw->key_root=info->s->state.key_root[ftb->keynr];
+    ftbw->keyinfo=info->s->keyinfo+ftb->keynr;
+    ftbw->off=0;
+    return _ft2_search(ftb, ftbw, 0);
+  }
+
+  /* matching key found */
+  memcpy(lastkey_buf, info->lastkey, info->lastkey_length);
+  if (lastkey_buf == ftbw->word)
+    ftbw->len=info->lastkey_length-extra;
+
+  /* going down ? */
+  if (subkeys<0)
+  {
+    /*
+      yep, going down, to the second-level tree
+      TODO here: subkey-based optimization
+    */
+    ftbw->off=off;
+    ftbw->key_root=info->lastpos;
+    ftbw->keyinfo=& info->s->ft2_keyinfo;
+    r=_mi_search_first(info, ftbw->keyinfo, ftbw->key_root);
+    DBUG_ASSERT(r==0);  /* found something */
+    memcpy(lastkey_buf+off, info->lastkey, info->lastkey_length);
+  }
+  ftbw->docid[0]=info->lastpos;
+  return 0;
+}
+
 static void _ftb_init_index_search(FT_INFO *ftb)
 {
-  int i, r;
+  int i;
   FTB_WORD   *ftbw;
-  MI_INFO    *info=ftb->info;
-  MI_KEYDEF  *keyinfo;
-  my_off_t    keyroot;
 
   if ((ftb->state != READY && ftb->state !=INDEX_DONE) ||
       ftb->keynr == NO_SUCH_KEY)
     return;
   ftb->state=INDEX_SEARCH;
 
-  keyinfo=info->s->keyinfo+ftb->keynr;
-  keyroot=info->s->state.key_root[ftb->keynr];
-
   for (i=ftb->queue.elements; i; i--)
   {
     ftbw=(FTB_WORD *)(ftb->queue.root[i]);
@@ -220,7 +333,7 @@ static void _ftb_init_index_search(FT_INFO *ftb)
     if (ftbw->flags & FTB_FLAG_TRUNC)
     {
       /*
-	special treatment for truncation operator
+        special treatment for truncation operator
         1. there are some (besides this) +words
            | no need to search in the index, it can never ADD new rows
            | to the result, and to remove half-matched rows we do scan anyway
@@ -244,11 +357,12 @@ static void _ftb_init_index_search(FT_INFO *ftb)
         if (ftbe->flags & FTB_FLAG_NO ||                     /* 2 */
              ftbe->up->ythresh - ftbe->up->yweaks >1)        /* 1 */
         {
-          FTB_EXPR *top_ftbe=ftbe->up->up;
+          FTB_EXPR *top_ftbe=ftbe->up;
           ftbw->docid[0]=HA_OFFSET_ERROR;
-          for (ftbe=ftbw->up; ftbe != top_ftbe; ftbe=ftbe->up)
-            if (!(ftbe->flags & FTB_FLAG_NO))
-              ftbe->yweaks++;
+          for (ftbe=(FTB_EXPR *)ftbw;
+               ftbe != top_ftbe && !(ftbe->flags & FTB_FLAG_NO);
+               ftbe=ftbe->up)
+              ftbe->up->yweaks++;
           ftbe=0;
           break;
         }
@@ -262,48 +376,17 @@ static void _ftb_init_index_search(FT_INFO *ftb)
       else
         reset_tree(& ftb->no_dupes);
     }
-    for (
-         r=_mi_search(info, keyinfo, (uchar*) ftbw->word, ftbw->len,
-                      SEARCH_FIND | SEARCH_BIGGER, keyroot) ;
-         !r && info->lastpos >= info->state->data_file_length;
-         r=_mi_search_next(info, keyinfo, info->lastkey, info->lastkey_length,
-                           SEARCH_BIGGER, keyroot)
-        );
 
-    if (!r)
-    {
-      r=_mi_compare_text(ftb->charset,
-                         info->lastkey + (ftbw->flags&FTB_FLAG_TRUNC),
-                         ftbw->len     - (ftbw->flags&FTB_FLAG_TRUNC),
-                         (uchar*) ftbw->word    + (ftbw->flags&FTB_FLAG_TRUNC),
-                         ftbw->len     - (ftbw->flags&FTB_FLAG_TRUNC),
-                         0);
-    }
-    if (r) /* not found */
-    {
-      if (ftbw->flags&FTB_FLAG_YES && ftbw->up->up==0)
-      {
-	/*
-	  This word MUST BE present in every document returned,
-	  so we can abort the search right now
-	*/
-        ftb->state=INDEX_DONE;
-        return;
-      }
-    }
-    else
-    {
-      memcpy(ftbw->word, info->lastkey, info->lastkey_length);
-      ftbw->docid[0]=info->lastpos;
-    }
+    ftbw->off=0; /* in case of reinit */
+    if (_ft2_search(ftb, ftbw, 1))
+      return;
   }
   queue_fix(& ftb->queue);
 }
 
 
 FT_INFO * ft_init_boolean_search(MI_INFO *info, uint keynr, byte *query,
-				 uint query_len,
-				 my_bool presort __attribute__((unused)))
+                                 uint query_len, CHARSET_INFO *cs)
 {
   FTB       *ftb;
   FTB_EXPR  *ftbe;
@@ -315,9 +398,8 @@ FT_INFO * ft_init_boolean_search(MI_INFO *info, uint keynr, byte *query,
   ftb->state=UNINITIALIZED;
   ftb->info=info;
   ftb->keynr=keynr;
-  ftb->charset= ((keynr==NO_SUCH_KEY) ?
-    default_charset_info              :
-    info->s->keyinfo[keynr].seg->charset);
+  ftb->charset=cs;
+  DBUG_ASSERT(keynr==NO_SUCH_KEY || cs == info->s->keyinfo[keynr].seg->charset);
   ftb->with_scan=0;
   ftb->lastpos=HA_OFFSET_ERROR;
   bzero(& ftb->no_dupes, sizeof(TREE));
@@ -328,7 +410,7 @@ FT_INFO * ft_init_boolean_search(MI_INFO *info, uint keynr, byte *query,
     Hack: instead of init_queue, we'll use reinit queue to be able
     to alloc queue with alloc_root()
   */
-  res=ftb->queue.max_elements=1+query_len/(min(ft_min_word_len,2)+1);
+  res=ftb->queue.max_elements=1+query_len/2;
   if (!(ftb->queue.root=
         (byte **)alloc_root(&ftb->mem_root, (res+1)*sizeof(void*))))
     goto err;
@@ -339,12 +421,12 @@ FT_INFO * ft_init_boolean_search(MI_INFO *info, uint keynr, byte *query,
   ftbe->weight=1;
   ftbe->flags=FTB_FLAG_YES;
   ftbe->nos=1;
-  ftbe->quot=0;
   ftbe->up=0;
   ftbe->ythresh=ftbe->yweaks=0;
   ftbe->docid[0]=ftbe->docid[1]=HA_OFFSET_ERROR;
+  ftbe->phrase= NULL;
   ftb->root=ftbe;
-  _ftb_parse_query(ftb, &query, query+query_len, ftbe, 0);
+  _ftb_parse_query(ftb, &query, query+query_len, ftbe, 0, NULL);
   ftb->list=(FTB_WORD **)alloc_root(&ftb->mem_root,
                                      sizeof(FTB_WORD *)*ftb->queue.elements);
   memcpy(ftb->list, ftb->queue.root+1, sizeof(FTB_WORD *)*ftb->queue.elements);
@@ -360,37 +442,46 @@ err:
 }
 
 
-/* returns 1 if str0 ~= /\bstr1\b/ */
-static int _ftb_strstr(const byte *s0, const byte *e0,
-                const byte *s1, const byte *e1,
-                CHARSET_INFO *cs)
+/*
+  Checks if given buffer matches phrase list.
+
+  SYNOPSIS
+    _ftb_check_phrase()
+    s0     start of buffer
+    e0     end of buffer
+    phrase broken into list phrase
+    cs     charset info
+
+  RETURN VALUE
+    1 is returned if phrase found, 0 else.
+*/
+
+static int _ftb_check_phrase(const byte *s0, const byte *e0,
+                LIST *phrase, CHARSET_INFO *cs)
 {
-  const byte *p0, *p1;
-  my_bool s_after, e_before;
+  FT_WORD h_word;
+  const byte *h_start= s0;
+  DBUG_ENTER("_ftb_strstr");
+  DBUG_ASSERT(phrase);
 
-  s_after=true_word_char(s1[0]);
-  e_before=true_word_char(e1[-1]);
-  p0=s0;
-
-  while (p0 < e0)
+  while (ft_simple_get_word(cs, (byte **)&h_start, e0, &h_word, FALSE))
   {
-    while (p0 < e0 && cs->to_upper[(uint) (uchar) *p0++] !=
-	   cs->to_upper[(uint) (uchar) *s1])
-      /* no-op */;
-    if (p0 >= e0)
-      return 0;
-
-    if (s_after && p0-1 > s0 && true_word_char(p0[-2]))
-      continue;
-
-    p1=s1+1;
-    while (p0 < e0 && p1 < e1 && cs->to_upper[(uint) (uchar) *p0] ==
-	   cs->to_upper[(uint) (uchar) *p1])
-      p0++, p1++;
-    if (p1 == e1 && (!e_before || p0 == e0 || !true_word_char(p0[0])))
-      return 1;
+    FT_WORD *n_word;
+    LIST *phrase_element= phrase;
+    const byte *h_start1= h_start;
+    for (;;)
+    {
+      n_word= (FT_WORD *)phrase_element->data;
+      if (my_strnncoll(cs, (const uchar *) h_word.pos, h_word.len,
+		       (const uchar *) n_word->pos, n_word->len))
+        break;
+      if (! (phrase_element= phrase_element->next))
+        DBUG_RETURN(1);
+      if (! ft_simple_get_word(cs, (byte **)&h_start1, e0, &h_word, FALSE))
+        DBUG_RETURN(0);
+    }
   }
-  return 0;
+  DBUG_RETURN(0);
 }
 
 
@@ -417,11 +508,11 @@ static void _ftb_climb_the_tree(FTB *ftb, FTB_WORD *ftbw, FT_SEG_ITERATOR *ftsi_
     {
       weight /= ftbe->ythresh;
       ftbe->cur_weight += weight;
-      if (++ftbe->yesses == ythresh)
+      if ((int) ++ftbe->yesses == ythresh)
       {
         yn=ftbe->flags;
         weight=ftbe->cur_weight*ftbe->weight;
-        if (mode && ftbe->quot)
+        if (mode && ftbe->phrase)
         {
           int not_found=1;
 
@@ -430,8 +521,8 @@ static void _ftb_climb_the_tree(FTB *ftb, FTB_WORD *ftbw, FT_SEG_ITERATOR *ftsi_
           {
             if (!ftsi.pos)
               continue;
-            not_found = ! _ftb_strstr(ftsi.pos, ftsi.pos+ftsi.len,
-                                      ftbe->quot, ftbe->qend, ftb->charset);
+            not_found = ! _ftb_check_phrase(ftsi.pos, ftsi.pos+ftsi.len,
+                                      ftbe->phrase, ftb->charset);
           }
           if (not_found) break;
         } /* ftbe->quot */
@@ -443,12 +534,12 @@ static void _ftb_climb_the_tree(FTB *ftb, FTB_WORD *ftbw, FT_SEG_ITERATOR *ftsi_
     if (yn & FTB_FLAG_NO)
     {
       /*
-	NOTE: special sort function of queue assures that all
-	(yn & FTB_FLAG_NO) != 0
-	events for every particular subexpression will
-	"auto-magically" happen BEFORE all the
-	(yn & FTB_FLAG_YES) != 0 events. So no
-	already matched expression can become not-matched again.
+        NOTE: special sort function of queue assures that all
+        (yn & FTB_FLAG_NO) != 0
+        events for every particular subexpression will
+        "auto-magically" happen BEFORE all the
+        (yn & FTB_FLAG_YES) != 0 events. So no
+        already matched expression can become not-matched again.
       */
       ++ftbe->nos;
       break;
@@ -456,12 +547,12 @@ static void _ftb_climb_the_tree(FTB *ftb, FTB_WORD *ftbw, FT_SEG_ITERATOR *ftsi_
     else
     {
       if (ftbe->ythresh)
-	weight/=3;
+        weight/=3;
       ftbe->cur_weight +=  weight;
-      if (ftbe->yesses < ythresh)
+      if ((int) ftbe->yesses < ythresh)
         break;
       if (!(yn & FTB_FLAG_WONLY))
-        yn= (ftbe->yesses++ == ythresh) ? ftbe->flags : FTB_FLAG_WONLY ;
+        yn= ((int) ftbe->yesses++ == ythresh) ? ftbe->flags : FTB_FLAG_WONLY ;
       weight*= ftbe->weight;
     }
   }
@@ -473,10 +564,7 @@ int ft_boolean_read_next(FT_INFO *ftb, char *record)
   FTB_EXPR  *ftbe;
   FTB_WORD  *ftbw;
   MI_INFO   *info=ftb->info;
-  MI_KEYDEF *keyinfo=info->s->keyinfo+ftb->keynr;
-  my_off_t   keyroot=info->s->state.key_root[ftb->keynr];
   my_off_t   curdoc;
-  int        r;
 
   if (ftb->state != INDEX_SEARCH && ftb->state != INDEX_DONE)
     return -1;
@@ -495,47 +583,15 @@ int ft_boolean_read_next(FT_INFO *ftb, char *record)
   ftb->queue.first_cmp_arg=(void *)&curdoc;
 
   while (ftb->state == INDEX_SEARCH &&
-	 (curdoc=((FTB_WORD *)queue_top(& ftb->queue))->docid[0]) !=
-	 HA_OFFSET_ERROR)
+         (curdoc=((FTB_WORD *)queue_top(& ftb->queue))->docid[0]) !=
+         HA_OFFSET_ERROR)
   {
-    while (curdoc==(ftbw=(FTB_WORD *)queue_top(& ftb->queue))->docid[0])
+    while (curdoc == (ftbw=(FTB_WORD *)queue_top(& ftb->queue))->docid[0])
     {
       _ftb_climb_the_tree(ftb, ftbw, 0);
 
       /* update queue */
-      for (
-          r=_mi_search(info, keyinfo, (uchar*) ftbw->word, USE_WHOLE_KEY,
-                       SEARCH_BIGGER, keyroot) ;
-           !r && info->lastpos >= info->state->data_file_length ;
-           r=_mi_search_next(info, keyinfo, info->lastkey, info->lastkey_length,
-                             SEARCH_BIGGER, keyroot)
-          );
-      if (!r)
-      {
-        r=_mi_compare_text(ftb->charset,
-                           info->lastkey + (ftbw->flags&FTB_FLAG_TRUNC),
-                           ftbw->len     - (ftbw->flags&FTB_FLAG_TRUNC),
-                           (uchar*) ftbw->word + (ftbw->flags&FTB_FLAG_TRUNC),
-                           ftbw->len     - (ftbw->flags&FTB_FLAG_TRUNC),
-                           0);
-      }
-      if (r) /* not found */
-      {
-        ftbw->docid[0]=HA_OFFSET_ERROR;
-        if (ftbw->flags&FTB_FLAG_YES && ftbw->up->up==0)
-        {
-	  /*
-	    This word MUST BE present in every document returned,
-	    so we can stop the search right now
-	  */
-          ftb->state=INDEX_DONE;
-        }
-      }
-      else
-      {
-        memcpy(ftbw->word, info->lastkey, info->lastkey_length);
-        ftbw->docid[0]=info->lastpos;
-      }
+      _ft2_search(ftb, ftbw, 0);
       queue_replaced(& ftb->queue);
     }
 
@@ -544,12 +600,14 @@ int ft_boolean_read_next(FT_INFO *ftb, char *record)
         ftbe->yesses>=(ftbe->ythresh-ftbe->yweaks) && !ftbe->nos)
     {
       /* curdoc matched ! */
-      if (is_tree_inited(& ftb->no_dupes) &&
-          tree_insert(& ftb->no_dupes, &curdoc, 0)->count >1)
-        /* but it managed to get past this line once */
+      if (is_tree_inited(&ftb->no_dupes) &&
+          tree_insert(&ftb->no_dupes, &curdoc, 0,
+                      ftb->no_dupes.custom_arg)->count >1)
+        /* but it managed already to get past this line once */
         continue;
 
       info->lastpos=curdoc;
+      /* Clear all states, except that the table was updated */
       info->update&= (HA_STATE_CHANGED | HA_STATE_ROW_CHANGED);
 
       if (!(*info->read_record)(info,curdoc,record))
@@ -612,15 +670,16 @@ float ft_boolean_find_relevance(FT_INFO *ftb, byte *record, uint length)
       continue;
 
     end=ftsi.pos+ftsi.len;
-    while (ft_simple_get_word((byte **) &ftsi.pos,(byte *) end, &word))
+    while (ft_simple_get_word(ftb->charset, (byte **) &ftsi.pos,
+                              (byte *) end, &word, TRUE))
     {
       int a, b, c;
       for (a=0, b=ftb->queue.elements, c=(a+b)/2; b-a>1; c=(a+b)/2)
       {
         ftbw=ftb->list[c];
-        if (_mi_compare_text(ftb->charset, (uchar*) word.pos, word.len,
-                             (uchar*) ftbw->word+1, ftbw->len-1,
-                             (my_bool) (ftbw->flags&FTB_FLAG_TRUNC)) >0)
+        if (mi_compare_text(ftb->charset, (uchar*) word.pos, word.len,
+                            (uchar*) ftbw->word+1, ftbw->len-1,
+                            (my_bool) (ftbw->flags&FTB_FLAG_TRUNC),0) >0)
           b=c;
         else
           a=c;
@@ -628,9 +687,9 @@ float ft_boolean_find_relevance(FT_INFO *ftb, byte *record, uint length)
       for (; c>=0; c--)
       {
         ftbw=ftb->list[c];
-        if (_mi_compare_text(ftb->charset, (uchar*) word.pos, word.len,
-                             (uchar*) ftbw->word+1,ftbw->len-1,
-                              (my_bool) (ftbw->flags&FTB_FLAG_TRUNC)))
+        if (mi_compare_text(ftb->charset, (uchar*) word.pos, word.len,
+                            (uchar*) ftbw->word+1,ftbw->len-1,
+                            (my_bool) (ftbw->flags&FTB_FLAG_TRUNC),0))
           break;
         if (ftbw->docid[1] == docid)
           continue;
