@@ -1,9 +1,8 @@
-/* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+/* Copyright (C) 2000-2004, 2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -71,6 +70,21 @@ ha_rows mi_records_in_range(MI_INFO *info, int inx, key_range *min_key,
     uchar * key_buff;
     uint start_key_len;
 
+    /*
+      The problem is that the optimizer doesn't support
+      RTree keys properly at the moment.
+      Hope this will be fixed some day.
+      But now NULL in the min_key means that we
+      didn't make the task for the RTree key
+      and expect BTree functionality from it.
+      As it's not able to handle such request
+      we return the error.
+    */
+    if (!min_key)
+    {
+      res= HA_POS_ERROR;
+      break;
+    }
     key_buff= info->lastkey+info->s->base.max_key_length;
     start_key_len= _mi_pack_key(info,inx, key_buff,
                                 (uchar*) min_key->key, min_key->length,
@@ -156,6 +170,7 @@ static double _mi_search_pos(register MI_INFO *info,
   uchar *keypos,*buff;
   double offset;
   DBUG_ENTER("_mi_search_pos");
+  LINT_INIT(max_keynr);
 
   if (pos == HA_OFFSET_ERROR)
     DBUG_RETURN(0.5);
@@ -172,9 +187,9 @@ static double _mi_search_pos(register MI_INFO *info,
     if (flag == MI_FOUND_WRONG_KEY)
       DBUG_RETURN(-1);				/* error */
     /*
-    ** Didn't found match. keypos points at next (bigger) key
-    *  Try to find a smaller, better matching key.
-    ** Matches keynr + [0-1]
+      Didn't found match. keypos points at next (bigger) key
+      Try to find a smaller, better matching key.
+      Matches keynr + [0-1]
     */
     if (flag > 0 && ! nod_flag)
       offset= 1.0;
@@ -185,8 +200,8 @@ static double _mi_search_pos(register MI_INFO *info,
   else
   {
     /*
-    ** Found match. Keypos points at the start of the found key
-    ** Matches keynr+1
+      Found match. Keypos points at the start of the found key
+      Matches keynr+1
     */
     offset=1.0;					/* Matches keynr+1 */
     if ((nextflag & SEARCH_FIND) && nod_flag &&
@@ -194,8 +209,8 @@ static double _mi_search_pos(register MI_INFO *info,
 	 key_len != USE_WHOLE_KEY))
     {
       /*
-      ** There may be identical keys in the tree. Try to match on of those.
-      ** Matches keynr + [0-1]
+        There may be identical keys in the tree. Try to match on of those.
+        Matches keynr + [0-1]
       */
       if ((offset=_mi_search_pos(info,keyinfo,key,key_len,SEARCH_FIND,
 				 _mi_kpos(nod_flag,keypos))) < 0)
@@ -213,7 +228,8 @@ err:
 
 	/* Get keynummer of current key and max number of keys in nod */
 
-static uint _mi_keynr(MI_INFO *info, register MI_KEYDEF *keyinfo, uchar *page, uchar *keypos, uint *ret_max_key)
+static uint _mi_keynr(MI_INFO *info, register MI_KEYDEF *keyinfo, uchar *page,
+                      uchar *keypos, uint *ret_max_key)
 {
   uint nod_flag,keynr,max_key;
   uchar t_buff[MI_MAX_KEY_BUFF],*end;
@@ -222,7 +238,7 @@ static uint _mi_keynr(MI_INFO *info, register MI_KEYDEF *keyinfo, uchar *page, u
   nod_flag=mi_test_if_nod(page);
   page+=2+nod_flag;
 
-  if (!(keyinfo->flag & (HA_VAR_LENGTH_KEY| HA_BINARY_PACK_KEY)))
+  if (!(keyinfo->flag & (HA_VAR_LENGTH_KEY | HA_BINARY_PACK_KEY)))
   {
     *ret_max_key= (uint) (end-page)/(keyinfo->keylength+nod_flag);
     return (uint) (keypos-page)/(keyinfo->keylength+nod_flag);
