@@ -1,9 +1,8 @@
-/* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+/* Copyright (C) 2000-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -68,6 +67,7 @@ int mi_rkey(MI_INFO *info, byte *buf, int inx, const byte *key, uint key_len,
 
   if (fast_mi_readinfo(info))
     goto err;
+
   if (share->concurrent_insert)
     rw_rdlock(&share->key_root_lock[inx]);
 
@@ -90,21 +90,45 @@ int mi_rkey(MI_INFO *info, byte *buf, int inx, const byte *key, uint key_len,
   case HA_KEY_ALG_BTREE:
   default:
     if (!_mi_search(info, keyinfo, key_buff, use_key_length,
-		  myisam_read_vec[search_flag], info->s->state.key_root[inx]))
+                    myisam_read_vec[search_flag], info->s->state.key_root[inx]))
     {
-      while (info->lastpos >= info->state->data_file_length)
+      /*
+        If we searching for a partial key (or using >, >=, < or <=) and
+        the data is outside of the data file, we need to continue searching
+        for the first key inside the data file
+      */
+      if (info->lastpos >= info->state->data_file_length &&
+          (search_flag != HA_READ_KEY_EXACT ||
+           last_used_keyseg != keyinfo->seg + keyinfo->keysegs))
       {
-        /*
-	  Skip rows that are inserted by other threads since we got a lock
-	  Note that this can only happen if we are not searching after an
-	  exact key, because the keys are sorted according to position
-        */
-
-        if  (_mi_search_next(info, keyinfo, info->lastkey,
-			   info->lastkey_length,
-			   myisam_readnext_vec[search_flag],
-			   info->s->state.key_root[inx]))
-	  break;
+        do
+        {
+          uint not_used[2];
+          /*
+            Skip rows that are inserted by other threads since we got a lock
+            Note that this can only happen if we are not searching after an
+            full length exact key, because the keys are sorted
+            according to position
+          */
+          if  (_mi_search_next(info, keyinfo, info->lastkey,
+                               info->lastkey_length,
+                               myisam_readnext_vec[search_flag],
+                               info->s->state.key_root[inx]))
+            break;
+          /*
+            Check that the found key does still match the search.
+            _mi_search_next() delivers the next key regardless of its
+            value.
+          */
+          if (search_flag == HA_READ_KEY_EXACT &&
+              ha_key_cmp(keyinfo->seg, key_buff, info->lastkey, use_key_length,
+                         SEARCH_FIND, not_used))
+          {
+            my_errno= HA_ERR_KEY_NOT_FOUND;
+            info->lastpos= HA_OFFSET_ERROR;
+            break;
+          }
+        } while (info->lastpos >= info->state->data_file_length);
       }
     }
   }
