@@ -1,4 +1,18 @@
 # -*- cperl -*-
+# Copyright (C) 2004-2006 MySQL AB
+# 
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; version 2 of the License.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 # This is a library file used by the Perl version of mysql-test-run,
 # and is part of the translation of the Bourne shell script with the
@@ -14,6 +28,7 @@ sub mtr_path_exists(@);
 sub mtr_script_exists(@);
 sub mtr_file_exists(@);
 sub mtr_exe_exists(@);
+sub mtr_exe_maybe_exists(@);
 sub mtr_copy_dir($$);
 sub mtr_same_opts($$);
 sub mtr_cmp_opts($$);
@@ -65,6 +80,10 @@ sub mtr_add_arg ($$@) {
 
 ##############################################################################
 
+#
+# NOTE! More specific paths should be given before less specific.
+# For example /client/debug should be listed before /client
+#
 sub mtr_path_exists (@) {
   foreach my $path ( @_ )
   {
@@ -80,10 +99,22 @@ sub mtr_path_exists (@) {
   }
 }
 
+
+#
+# NOTE! More specific paths should be given before less specific.
+# For example /client/debug should be listed before /client
+#
 sub mtr_script_exists (@) {
   foreach my $path ( @_ )
   {
-    return $path if -x $path;
+    if($::glob_win32)
+    {
+      return $path if -f $path;
+    }
+    else
+    {
+      return $path if -x $path;
+    }
   }
   if ( @_ == 1 )
   {
@@ -95,6 +126,11 @@ sub mtr_script_exists (@) {
   }
 }
 
+
+#
+# NOTE! More specific paths should be given before less specific.
+# For example /client/debug should be listed before /client
+#
 sub mtr_file_exists (@) {
   foreach my $path ( @_ )
   {
@@ -103,13 +139,42 @@ sub mtr_file_exists (@) {
   return "";
 }
 
-sub mtr_exe_exists (@) {
+
+#
+# NOTE! More specific paths should be given before less specific.
+# For example /client/debug should be listed before /client
+#
+sub mtr_exe_maybe_exists (@) {
   my @path= @_;
+
   map {$_.= ".exe"} @path if $::glob_win32;
+  map {$_.= ".nlm"} @path if $::glob_netware;
   foreach my $path ( @path )
   {
-    return $path if -x $path;
+    if($::glob_win32)
+    {
+      return $path if -f $path;
+    }
+    else
+    {
+      return $path if -x $path;
+    }
   }
+  return "";
+}
+
+
+#
+# NOTE! More specific paths should be given before less specific.
+# For example /client/debug should be listed before /client
+#
+sub mtr_exe_exists (@) {
+  my @path= @_;
+  if (my $path= mtr_exe_maybe_exists(@path))
+  {
+    return $path;
+  }
+  # Could not find exe, show error
   if ( @path == 1 )
   {
     mtr_error("Could not find $path[0]");
@@ -125,6 +190,8 @@ sub mtr_copy_dir($$) {
   my $from_dir= shift;
   my $to_dir= shift;
 
+  # mtr_verbose("Copying from $from_dir to $to_dir");
+
   mkpath("$to_dir");
   opendir(DIR, "$from_dir")
     or mtr_error("Can't find $from_dir$!");
