@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,9 +29,9 @@ File my_dup(File file, myf MyFlags)
   File fd;
   const char *filename;
   DBUG_ENTER("my_dup");
-  DBUG_PRINT("my",("file: %d  MyFlags: %d", MyFlags));
+  DBUG_PRINT("my",("file: %d  MyFlags: %d", file, MyFlags));
   fd = dup(file);
-  filename= (((int) file < MY_NFILE) ?
+  filename= (((uint) file < my_file_limit) ?
 	     my_file_info[(int) file].name : "Unknown");
   DBUG_RETURN(my_register_filename(fd, filename, FILE_BY_DUP,
 				   EE_FILENOTFOUND, MyFlags));
