@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,13 +16,8 @@
 #include "mysys_priv.h"
 #include "my_static.h"
 #include "mysys_err.h"
-#include "m_ctype.h"
 #include <m_string.h>
 #include <m_ctype.h>
-#ifdef HAVE_GETRUSAGE
-#include <sys/resource.h>
-/* extern int     getrusage(int, struct rusage *); */
-#endif
 #include <signal.h>
 #ifdef VMS
 #include <my_static.c>
@@ -47,16 +41,16 @@ static void netware_init();
 #define netware_init()
 #endif
 
-
-my_bool my_init_done=0;
+my_bool my_init_done= 0;
+uint	mysys_usage_id= 0;              /* Incremented for each my_init() */
 
 static ulong atoi_octal(const char *str)
 {
   long int tmp;
-  while (*str && isspace(*str))
+  while (*str && my_isspace(&my_charset_latin1, *str))
     str++;
   str2int(str,
-	  (*str == '0' ? 8 : 10),		/* Octalt or decimalt */
+	  (*str == '0' ? 8 : 10),       /* Octalt or decimalt */
 	  0, INT_MAX, &tmp);
   return (ulong) tmp;
 }
@@ -79,6 +73,9 @@ my_bool my_init(void)
   if (my_init_done)
     return 0;
   my_init_done=1;
+  mysys_usage_id++;
+  my_umask= 0660;                       /* Default umask for new files */
+  my_umask_dir= 0700;                   /* Default umask for new directories */
 #if defined(THREAD) && defined(SAFE_MUTEX)
   safe_mutex_global_init();		/* Must be called early */
 #endif
@@ -98,7 +95,7 @@ my_bool my_init(void)
 #endif
   {
     DBUG_ENTER("my_init");
-    DBUG_PROCESS(my_progname ? my_progname : (char*) "unknown");
+    DBUG_PROCESS((char*) (my_progname ? my_progname : "unknown"));
     if (!home_dir)
     {					/* Don't initialize twice */
       my_win_init();
@@ -129,10 +126,23 @@ my_bool my_init(void)
 
 void my_end(int infoflag)
 {
-  FILE *info_file;
-  if (!(info_file=DBUG_FILE))
-    info_file=stderr;
-  if (infoflag & MY_CHECK_ERROR || info_file != stderr)
+  /*
+    this code is suboptimal to workaround a bug in
+    Sun CC: Sun C++ 5.6 2004/06/02 for x86, and should not be
+    optimized until this compiler is not in use anymore
+  */
+  FILE *info_file= DBUG_FILE;
+  my_bool print_info= (info_file != stderr);
+  DBUG_ENTER("my_end");
+  if (!info_file)
+  {
+    info_file= stderr;
+    print_info= 0;
+  }
+
+  DBUG_PRINT("info",("Shutting down: print_info: %d", print_info));
+  if ((infoflag & MY_CHECK_ERROR) || print_info)
+
   {					/* Test if some file is left open */
     if (my_file_opened | my_stream_opened)
     {
@@ -142,10 +152,16 @@ void my_end(int infoflag)
     }
   }
   free_charsets();
-  if (infoflag & MY_GIVE_INFO || info_file != stderr)
+  my_once_free();
+
+  if ((infoflag & MY_GIVE_INFO) || print_info)
   {
 #ifdef HAVE_GETRUSAGE
     struct rusage rus;
+#ifdef HAVE_purify
+    /* Purify assumes that rus is uninitialized after getrusage call */
+    bzero((char*) &rus, sizeof(rus));
+#endif
     if (!getrusage(RUSAGE_SELF, &rus))
       fprintf(info_file,"\n\
 User time %.2f, System time %.2f\n\
@@ -179,9 +195,10 @@ Voluntary context switches %ld, Involuntary context switches %ld\n",
    _CrtDumpMemoryLeaks();
 #endif
   }
+
+  if (!(infoflag & MY_DONT_FREE_DBUG))
+    DBUG_END();                /* Must be done before my_thread_end */
 #ifdef THREAD
-  DBUG_POP();				/* Must be done before my_thread_end */
-  my_once_free();
   my_thread_end();
   my_thread_global_end();
 #if defined(SAFE_MUTEX)
@@ -227,6 +244,22 @@ void setEnvString(char *ret, const char *name, const char *value)
   DBUG_VOID_RETURN ;
 }
 
+/*
+  my_paramter_handler
+  Invalid paramter handler we will use instead of the one "baked" into the CRT
+  for MSC v8.  This one just prints out what invalid parameter was encountered.
+  By providing this routine, routines like lseek will return -1 when we expect them 
+  to instead of crash.
+*/
+void my_parameter_handler(const wchar_t * expression, const wchar_t * function, 
+                          const wchar_t * file, unsigned int line, 
+                          uintptr_t pReserved)
+{
+  DBUG_PRINT("my",("Expression: %s  function: %s  file: %s, line: %d",
+		   expression, function, file, line));
+}
+
+
 static void my_win_init(void)
 {
   HKEY	hSoftMysql ;
@@ -244,8 +277,19 @@ static void my_win_init(void)
 
   setlocale(LC_CTYPE, "");             /* To get right sortorder */
 
-  /* Clear the OS system variable TZ and avoid the 100% CPU usage */
+#if defined(_MSC_VER)
+#if _MSC_VER < 1300
+  /* 
+    Clear the OS system variable TZ and avoid the 100% CPU usage
+    Only for old versions of Visual C++
+  */
   _putenv( "TZ=" ); 
+#endif 
+#if _MSC_VER >= 1400
+  /* this is required to make crt functions return -1 appropriately */
+  _set_invalid_parameter_handler(my_parameter_handler);
+#endif
+#endif  
   _tzset();
 
   /* apre la chiave HKEY_LOCAL_MACHINES\software\MySQL */
@@ -388,7 +432,7 @@ static void netware_init()
     }
 
     /* Parse program name and change to base format */
-    name= my_progname;
+    name= (char*) my_progname;
     for (; *name; name++)
     {
       if (*name == '\\')
