@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -638,7 +637,7 @@ MY_STAT *my_stat(const char *path, MY_STAT *stat_area, myf my_flags)
   int m_used;
   DBUG_ENTER("my_stat");
   DBUG_PRINT("my", ("path: '%s', stat_area: 0x%lx, MyFlags: %d", path,
-	     (byte *) stat_area, my_flags));
+                    (long) stat_area, my_flags));
 
   if ((m_used= (stat_area == NULL)))
     if (!(stat_area = (MY_STAT *) my_malloc(sizeof(MY_STAT), my_flags)))
