@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -335,7 +334,7 @@ File my_sopen(const char *path, int oflag, int shflag, int pmode)
    * try to open/create the file
    */
   if ((osfh= CreateFile(path, fileaccess, fileshare, &SecurityAttributes, 
-                        filecreate, fileattrib, NULL)) == (HANDLE)0xffffffff)
+                        filecreate, fileattrib, NULL)) == INVALID_HANDLE_VALUE)
   {
     /*
      * OS call to open/create file failed! map the error, release
@@ -346,7 +345,7 @@ File my_sopen(const char *path, int oflag, int shflag, int pmode)
     return -1;                      /* return error to caller */
   }
 
-  fh= _open_osfhandle((long)osfh, oflag & (_O_APPEND | _O_RDONLY | _O_TEXT));
+  fh= _open_osfhandle((intptr_t)osfh, oflag & (_O_APPEND | _O_RDONLY | _O_TEXT));
 
   return fh;                        /* return handle */
 }
