@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,10 +18,10 @@
 #include <my_dir.h>
 #include <m_string.h>
 #include "mysys_err.h"
-#if defined(HAVE_SYS_UTIME_H)
-#include <sys/utime.h>
-#elif defined(HAVE_UTIME_H)
+#if defined(HAVE_UTIME_H)
 #include <utime.h>
+#elif defined(HAVE_SYS_UTIME_H)
+#include <sys/utime.h>
 #elif !defined(HPUX10)
 struct utimbuf {
   time_t actime;
