@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -91,7 +90,7 @@ my_bool dynstr_realloc(DYNAMIC_STRING *str, ulong additional_size)
 
 my_bool dynstr_append(DYNAMIC_STRING *str, const char *append)
 {
-  return dynstr_append_mem(str,append,strlen(append));
+  return dynstr_append_mem(str,append,(uint) strlen(append));
 }
 
 
@@ -116,6 +115,56 @@ my_bool dynstr_append_mem(DYNAMIC_STRING *str, const char *append,
 }
 
 
+/** Concatenates any number of strings, escapes any OS quote in the result then
+ * surround the whole affair in another set of quotes which is finally appended
+ * to specified DYNAMIC_STRING.  This function is especially useful when 
+ * building strings to be executed with the system() function.
+ * @param str Dynamic String which will have addtional strings appended.
+ * @param append String to be appended.
+ * @param ... Optional. Additional string(s) to be appended.
+ *
+ * @note The final argument in the list must be NullS even if no additional
+ * options are passed.
+ *
+ * @return True = Success.
+ */
+my_bool dynstr_append_os_quoted(DYNAMIC_STRING *str, const char *append, ...)
+{
+#ifdef __WIN__
+  const char *quote_str= "\"";
+  const uint  quote_len= 1;
+#else
+  const char *quote_str= "\'";
+  const uint  quote_len= 1;
+#endif /* __WIN__ */
+  my_bool ret= TRUE;
+  va_list dirty_text;
+
+  ret&= dynstr_append_mem(str, quote_str, quote_len); /* Leading quote */
+  va_start(dirty_text, append);
+  while (append != NullS)
+  {
+    const char  *cur_pos= append;
+    const char *next_pos= cur_pos;
+
+    /* Search for quote in each string and replace with escaped quote */
+    while(*(next_pos= strcend(cur_pos, quote_str[0])) != '\0')
+    {
+      ret&= dynstr_append_mem(str, cur_pos, next_pos - cur_pos);
+      ret&= dynstr_append_mem(str ,"\\", 1);
+      ret&= dynstr_append_mem(str, quote_str, quote_len);
+      cur_pos= next_pos + 1;
+    }
+    ret&= dynstr_append_mem(str, cur_pos, next_pos - cur_pos);
+    append= va_arg(dirty_text, char *);
+  }
+  va_end(dirty_text);
+  ret&= dynstr_append_mem(str, quote_str, quote_len); /* Trailing quote */
+
+  return ret;
+}
+
+
 void dynstr_free(DYNAMIC_STRING *str)
 {
   if (str->str)
