@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -127,6 +126,7 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 /* 68 unused */
 /* 69 unused */
 /* 70 unused */
+#define GSN_UPDATE_FRAG_DIST_KEY_ORD    70
 #define GSN_ACC_ABORTREQ                71
 #define GSN_ACC_CHECK_SCAN              72
 #define GSN_ACC_COMMITCONF              73
@@ -587,6 +587,7 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_BLOCK_COMMIT_ORD            485
 #define GSN_UNBLOCK_COMMIT_ORD          486
 
+#define GSN_NODE_START_REP              502
 #define GSN_NODE_STATE_REP              487
 #define GSN_CHANGE_NODE_STATE_REQ       488
 #define GSN_CHANGE_NODE_STATE_CONF      489
@@ -731,9 +732,11 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_SUB_CREATE_REQ              576
 #define GSN_SUB_CREATE_REF              577
 #define GSN_SUB_CREATE_CONF             578
+/*
 #define GSN_SUB_START_REQ               579
 #define GSN_SUB_START_REF               580
 #define GSN_SUB_START_CONF              581
+*/
 #define GSN_SUB_SYNC_REQ                582
 #define GSN_SUB_SYNC_REF                583
 #define GSN_SUB_SYNC_CONF               584
@@ -899,10 +902,11 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 /**
  * SUMA restart protocol
  */
+/*
 #define GSN_SUMA_START_ME               691
 #define GSN_SUMA_HANDOVER_REQ           692
 #define GSN_SUMA_HANDOVER_CONF          693
-
+*/
 /* not used                             694 */
 /* not used                             695 */
 /* not used                             696 */
@@ -919,6 +923,7 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 /*
  * EVENT Signals
  */
+/*
 #define GSN_SUB_GCP_COMPLETE_ACC        699
 
 #define GSN_CREATE_EVNT_REQ             700
@@ -928,7 +933,7 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_DROP_EVNT_REQ               703
 #define GSN_DROP_EVNT_CONF              704
 #define GSN_DROP_EVNT_REF               705
-
+*/
 #define GSN_TUX_BOUND_INFO		710
 
 #define GSN_ACC_LOCKREQ			711
