@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -937,165 +936,6 @@ public:
     Index(NdbIndexImpl&);
   };
 
-  /**
-   * @brief Represents an Event in NDB Cluster
-   *
-   */
-  class Event : public Object  {
-  public:
-    /**
-     * Specifies the type of database operations an Event listens to
-     */
-#ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
-    /** TableEvent must match 1 << TriggerEvent */
-#endif
-    enum TableEvent { 
-      TE_INSERT=1, ///< Insert event on table
-      TE_DELETE=2, ///< Delete event on table
-      TE_UPDATE=4, ///< Update event on table
-      TE_ALL=7     ///< Any/all event on table (not relevant when 
-                   ///< events are received)
-    };
-    /**
-     *  Specifies the durability of an event
-     * (future version may supply other types)
-     */
-    enum EventDurability { 
-      ED_UNDEFINED
-#ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
-      = 0
-#endif
-#if 0 // not supported
-      ,ED_SESSION = 1, 
-      // Only this API can use it
-      // and it's deleted after api has disconnected or ndb has restarted
-      
-      ED_TEMPORARY = 2
-      // All API's can use it,
-      // But's its removed when ndb is restarted
-#endif
-      ,ED_PERMANENT    ///< All API's can use it.
-                       ///< It's still defined after a cluster system restart
-#ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
-      = 3
-#endif
-    };
-
-    /**
-     *  Constructor
-     *  @param  name  Name of event
-     */
-    Event(const char *name);
-    /**
-     *  Constructor
-     *  @param  name  Name of event
-     *  @param  table Reference retrieved from NdbDictionary
-     */
-    Event(const char *name, const NdbDictionary::Table& table);
-    virtual ~Event();
-    /**
-     * Set unique identifier for the event
-     */
-    void setName(const char *name);
-    /**
-     * Get unique identifier for the event
-     */
-    const char *getName() const;
-    /**
-     * Define table on which events should be detected
-     *
-     * @note calling this method will default to detection
-     *       of events on all columns. Calling subsequent
-     *       addEventColumn calls will override this.
-     *
-     * @param table reference retrieved from NdbDictionary
-     */
-    void setTable(const NdbDictionary::Table& table);
-    /**
-     * Set table for which events should be detected
-     *
-     * @note preferred way is using setTable(const NdbDictionary::Table&)
-     *       or constructor with table object parameter
-     */
-    void setTable(const char *tableName);
-    /**
-     * Get table name for events
-     *
-     * @return table name
-     */
-    const char* getTableName() const;
-    /**
-     * Add type of event that should be detected
-     */
-    void addTableEvent(const TableEvent te);
-    /**
-     * Set durability of the event
-     */
-    void setDurability(EventDurability);
-    /**
-     * Get durability of the event
-     */
-    EventDurability getDurability() const;
-#ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
-    void addColumn(const Column &c);
-#endif
-    /**
-     * Add a column on which events should be detected
-     *
-     * @param attrId Column id
-     *
-     * @note errors will mot be detected until createEvent() is called
-     */
-    void addEventColumn(unsigned attrId);
-    /**
-     * Add a column on which events should be detected
-     *
-     * @param columnName Column name
-     *
-     * @note errors will not be detected until createEvent() is called
-     */
-    void addEventColumn(const char * columnName);
-    /**
-     * Add several columns on which events should be detected
-     *
-     * @param n Number of columns
-     * @param columnNames Column names
-     *
-     * @note errors will mot be detected until 
-     *       NdbDictionary::Dictionary::createEvent() is called
-     */
-    void addEventColumns(int n, const char ** columnNames);
-
-    /**
-     * Get no of columns defined in an Event
-     *
-     * @return Number of columns, -1 on error
-     */
-    int getNoOfEventColumns() const;
-
-    /**
-     * Get object status
-     */
-    virtual Object::Status getObjectStatus() const;
-
-    /**
-     * Get object version
-     */
-    virtual int getObjectVersion() const;
-
-#ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
-    void print();
-#endif
-
-  private:
-#ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
-    friend class NdbEventImpl;
-    friend class NdbEventOperationImpl;
-#endif
-    class NdbEventImpl & m_impl;
-    Event(NdbEventImpl&);
-  };
-
   /**
    * @class Dictionary
    * @brief Dictionary for defining and retreiving meta data
@@ -1214,33 +1054,6 @@ public:
     int listIndexes(List & list, const char * tableName);
     int listIndexes(List & list, const char * tableName) const;
 
-    /** @} *******************************************************************/
-    /** 
-     * @name Events
-     * @{
-     */
-    
-    /**
-     * Create event given defined Event instance
-     * @param event Event to create
-     * @return 0 if successful otherwise -1.
-     */
-    int createEvent(const Event &event);
-
-    /**
-     * Drop event with given name
-     * @param eventName  Name of event to drop.
-     * @return 0 if successful otherwise -1.
-     */
-    int dropEvent(const char * eventName);
-    
-    /**
-     * Get event with given name.
-     * @param eventName  Name of event to get.
-     * @return an Event if successful, otherwise NULL.
-     */
-    const Event * getEvent(const char * eventName);
-
     /** @} *******************************************************************/
 
     /** 
