@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,19 +33,21 @@ SCI_Transporter::SCI_Transporter(TransporterRegistry &t_reg,
                                  const char *lHostName,
                                  const char *rHostName,
                                  int r_port,
+				 bool isMgmConnection,
                                  Uint32 packetSize,         
 				 Uint32 bufferSize,       
 				 Uint32 nAdapters, 
 				 Uint16 remoteSciNodeId0,        
 				 Uint16 remoteSciNodeId1, 
 				 NodeId _localNodeId,      
-				 NodeId _remoteNodeId,     
+				 NodeId _remoteNodeId,
+				 NodeId serverNodeId,
 				 bool chksm,  
 				 bool signalId, 
 				 Uint32 reportFreq) :  
   Transporter(t_reg, tt_SCI_TRANSPORTER,
-	      lHostName, rHostName, r_port, _localNodeId,
-              _remoteNodeId, 0, false, chksm, signalId) 
+	      lHostName, rHostName, r_port, isMgmConnection, _localNodeId,
+              _remoteNodeId, serverNodeId, 0, false, chksm, signalId) 
 {
   DBUG_ENTER("SCI_Transporter::SCI_Transporter");
   m_PacketSize = (packetSize + 3)/4 ; 
