@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -91,6 +90,7 @@ Cmvmi::Cmvmi(const Configuration & conf) :
   addRecSignal(GSN_DUMP_STATE_ORD, &Cmvmi::execDUMP_STATE_ORD);
 
   addRecSignal(GSN_TESTSIG, &Cmvmi::execTESTSIG);
+  addRecSignal(GSN_NODE_START_REP, &Cmvmi::execNODE_START_REP, true);
   
   subscriberPool.setSize(5);
 
@@ -340,6 +340,16 @@ void Cmvmi::execSTTOR(Signal* signal)
   jamEntry();
   if (theStartPhase == 1){
     jam();
+
+    if(theConfig.lockPagesInMainMemory() == 1)
+    {
+      int res = NdbMem_MemLockAll(0);
+      if(res != 0){
+	g_eventLogger.warning("Failed to memlock pages");
+	warningEvent("Failed to memlock pages");
+      }
+    }
+    
     sendSTTORRY(signal);
     return;
   } else if (theStartPhase == 3) {
@@ -359,18 +369,6 @@ void Cmvmi::execSTTOR(Signal* signal)
     signal->theData[2] = NodeInfo::REP;
     execOPEN_COMREQ(signal);    
     globalData.theStartLevel = NodeState::SL_STARTED;
-    sendSTTORRY(signal);
-  } else {
-    jam();
-
-    if(theConfig.lockPagesInMainMemory()){
-      int res = NdbMem_MemLockAll();
-      if(res != 0){
-	g_eventLogger.warning("Failed to memlock pages");
-	warningEvent("Failed to memlock pages");
-      }
-    }
-    
     sendSTTORRY(signal);
   }
 }
@@ -425,7 +423,8 @@ void Cmvmi::execOPEN_COMREQ(Signal* signal)
   if(len == 2){
 
 #ifdef ERROR_INSERT
-    if (! (ERROR_INSERTED(9000) && c_error_9000_nodes_mask.get(tStartingNode)))
+    if (! ((ERROR_INSERTED(9000) || ERROR_INSERTED(9002)) 
+	   && c_error_9000_nodes_mask.get(tStartingNode)))
 #endif
     {
       globalTransporterRegistry.do_connect(tStartingNode);
@@ -446,7 +445,8 @@ void Cmvmi::execOPEN_COMREQ(Signal* signal)
 	jam();
 
 #ifdef ERROR_INSERT
-	if (ERROR_INSERTED(9000) && c_error_9000_nodes_mask.get(i))
+	if ((ERROR_INSERTED(9000) || ERROR_INSERTED(9002))
+	    && c_error_9000_nodes_mask.get(i))
 	  continue;
 #endif
 	
@@ -697,6 +697,7 @@ Cmvmi::execTEST_ORD(Signal * signal){
       // Do nothing
       break;
     }
+    globalSignalLoggers.flushSignalLog();
   }
 
 #endif
@@ -787,6 +788,21 @@ Cmvmi::execSTART_ORD(Signal* signal) {
   
   if(globalData.theStartLevel == NodeState::SL_CMVMI){
     jam();
+
+    if(theConfig.lockPagesInMainMemory() == 2)
+    {
+      int res = NdbMem_MemLockAll(1);
+      if(res != 0)
+      {
+	g_eventLogger.warning("Failed to memlock pages");
+	warningEvent("Failed to memlock pages");
+      }
+      else
+      {
+	g_eventLogger.info("Locked future allocations");
+      }
+    }
+    
     globalData.theStartLevel  = NodeState::SL_STARTING;
     globalData.theRestartFlag = system_started;
     /**
@@ -1144,9 +1160,9 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
   }
 
 #ifdef ERROR_INSERT
-  if (arg == 9000)
+  if (arg == 9000 || arg == 9002)
   {
-    SET_ERROR_INSERT_VALUE(9000);
+    SET_ERROR_INSERT_VALUE(arg);
     for (Uint32 i = 1; i<signal->getLength(); i++)
       c_error_9000_nodes_mask.set(signal->theData[i]);
   }
@@ -1193,6 +1209,17 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
 #endif
 }//Cmvmi::execDUMP_STATE_ORD()
 
+void
+Cmvmi::execNODE_START_REP(Signal* signal)
+{
+#ifdef ERROR_INSERT
+  if (ERROR_INSERTED(9002) && signal->theData[0] == getOwnNodeId())
+  {
+    signal->theData[0] = 9001;
+    execDUMP_STATE_ORD(signal);
+  }
+#endif
+}
 
 BLOCK_FUNCTIONS(Cmvmi)
 
@@ -1234,7 +1261,7 @@ Cmvmi::execTESTSIG(Signal* signal){
     fprintf(stdout, "\n");
     
     for(i = 0; i<signal->header.m_noOfSections; i++){
-      SegmentedSectionPtr ptr;
+      SegmentedSectionPtr ptr = {0,0,0};
       ndbout_c("-- Section %d --", i);
       signal->getSection(ptr, i);
       ndbrequire(ptr.p != 0);
@@ -1292,7 +1319,7 @@ Cmvmi::execTESTSIG(Signal* signal){
     LinearSectionPtr ptr[3];
     const Uint32 secs = signal->getNoOfSections();
     for(i = 0; i<secs; i++){
-      SegmentedSectionPtr sptr;
+      SegmentedSectionPtr sptr = {0,0,0};
       signal->getSection(sptr, i);
       ptr[i].sz = sptr.sz;
       ptr[i].p = new Uint32[sptr.sz];
@@ -1341,7 +1368,7 @@ Cmvmi::execTESTSIG(Signal* signal){
     LinearSectionPtr ptr[3];
     const Uint32 secs = signal->getNoOfSections();
     for(i = 0; i<secs; i++){
-      SegmentedSectionPtr sptr;
+      SegmentedSectionPtr sptr = {0,0,0};
       signal->getSection(sptr, i);
       ptr[i].sz = sptr.sz;
       ptr[i].p = new Uint32[sptr.sz];
@@ -1407,7 +1434,7 @@ Cmvmi::execTESTSIG(Signal* signal){
     const Uint32 secs = signal->getNoOfSections();
     memset(g_test, 0, sizeof(g_test));
     for(i = 0; i<secs; i++){
-      SegmentedSectionPtr sptr;
+      SegmentedSectionPtr sptr  = {0,0,0};
       signal->getSection(sptr, i);
       g_test[i].sz = sptr.sz;
       g_test[i].p = new Uint32[sptr.sz];
