@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1048,6 +1047,8 @@ private:
   void removeStoredReplica(FragmentstorePtr regFragptr,
                            ReplicaRecordPtr replicaPtr);
   void searchStoredReplicas(FragmentstorePtr regFragptr);
+  bool setup_create_replica(FragmentstorePtr, CreateReplicaRecord*,
+			    ConstPtr<ReplicaRecord>);
   void updateNodeInfo(FragmentstorePtr regFragptr);
 
 //------------------------------------
@@ -1599,6 +1600,9 @@ private:
    */
   void startInfoReply(Signal *, Uint32 nodeId);
 
+  // DIH specifics for execNODE_START_REP (sendDictUnlockOrd)
+  void execNODE_START_REP(Signal* signal);
+
   /*
    * Lock master DICT.  Only current use is by starting node
    * during NR.  A pool of slave records is convenient anyway.
