@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -397,12 +396,12 @@ void Dbtup::allocMoreFragPages(Fragrecord* const regFragPtr)
   Uint32 noAllocPages = regFragPtr->noOfPagesToGrow >> 3; // 12.5%
   noAllocPages += regFragPtr->noOfPagesToGrow >> 4; // 6.25%
   noAllocPages += 2;
-  regFragPtr->noOfPagesToGrow += noAllocPages;
 /* -----------------------------------------------------------------*/
 // We will grow by 18.75% plus two more additional pages to grow
 // a little bit quicker in the beginning.
 /* -----------------------------------------------------------------*/
-  allocFragPages(regFragPtr, noAllocPages);
+  Uint32 allocated = allocFragPages(regFragPtr, noAllocPages);
+  regFragPtr->noOfPagesToGrow += allocated;
 }//Dbtup::allocMoreFragPages()
 
 Uint32 Dbtup::leafPageRangeFull(Fragrecord*  const regFragPtr, PageRangePtr currPageRangePtr)
