@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -181,10 +180,9 @@ void Ndbcntr::execSYSTEM_ERROR(Signal* signal)
 
   case SystemError::CopyFragRefError:
     BaseString::snprintf(buf, sizeof(buf), 
-	     "Node %d killed this node because "
-	     "it could not copy a fragment during node restart. "
-	     "Copy fragment error code: %u.",
-	     killingNode, data1);
+			 "Killed by node %d as "
+			 "copyfrag failed, error: %u",
+			 killingNode, data1);
     break;
 
   default:
@@ -591,6 +589,13 @@ Ndbcntr::execCNTR_START_REP(Signal* signal){
   Uint32 nodeId = signal->theData[0];
   c_startedNodes.set(nodeId);
   c_start.m_starting.clear(nodeId);
+
+  /**
+   * Inform all interested blocks that node has started
+   */
+  for(Uint32 i = 0; i<ALL_BLOCKS_SZ; i++){
+    sendSignal(ALL_BLOCKS[i].Ref, GSN_NODE_START_REP, signal, 1, JBB);
+  }
   
   if(!c_start.m_starting.isclear()){
     jam();
@@ -1461,9 +1466,6 @@ void Ndbcntr::execNODE_FAILREP(Signal* signal)
   sendSignal(BACKUP_REF, GSN_NODE_FAILREP, signal,
 	     NodeFailRep::SignalLength, JBB);
 
-  sendSignal(SUMA_REF, GSN_NODE_FAILREP, signal,
-	     NodeFailRep::SignalLength, JBB);
-
   if (c_stopRec.stopReq.senderRef)
   {
     jam();
@@ -1521,6 +1523,11 @@ void Ndbcntr::execNODE_FAILREP(Signal* signal)
       
       break;
     }
+    case StopRecord::SR_BLOCK_GCP_START_GCP:
+    case StopRecord::SR_WAIT_COMPLETE_GCP:
+    case StopRecord::SR_UNBLOCK_GCP_START_GCP:
+    case StopRecord::SR_CLUSTER_SHUTDOWN:
+      break;
     }
   }
   
@@ -2039,6 +2046,11 @@ void Ndbcntr::execSET_VAR_REQ(Signal* signal) {
 void Ndbcntr::updateNodeState(Signal* signal, const NodeState& newState) const{
   NodeStateRep * const stateRep = (NodeStateRep *)&signal->theData[0];
 
+  if (newState.startLevel == NodeState::SL_STARTED)
+  {
+    CRASH_INSERTION(1000);
+  }
+
   stateRep->nodeState = newState;
   stateRep->nodeState.masterNodeId = cmasterNodeId;
   stateRep->nodeState.setNodeGroup(c_nodeGroup);
@@ -2265,7 +2277,33 @@ Ndbcntr::StopRecord::checkNodeFail(Signal* signal){
   {
     NdbNodeBitmask tmp;
     tmp.assign(NdbNodeBitmask::Size, stopReq.nodes);
+
+    NdbNodeBitmask ndbStopNodes;
+    ndbStopNodes.assign(NdbNodeBitmask::Size, stopReq.nodes);
+    ndbStopNodes.bitAND(ndbMask);
+    ndbStopNodes.copyto(NdbNodeBitmask::Size, stopReq.nodes);
+
     ndbMask.bitANDC(tmp);
+
+    bool allNodesStopped = true;
+    int i ;
+    for( i = 0; i < (int) NdbNodeBitmask::Size; i++ ){
+      if ( stopReq.nodes[i] != 0 ){
+        allNodesStopped = false;
+        break;
+      }
+    }
+  
+    if ( allNodesStopped ) {
+      StopConf * const stopConf = (StopConf *)&signal->theData[0];
+      stopConf->senderData = stopReq.senderData;
+      stopConf->nodeState  = (Uint32) NodeState::SL_NOTHING;
+      cntr.sendSignal(stopReq.senderRef, GSN_STOP_CONF, signal,
+                       StopConf::SignalLength, JBB);
+      stopReq.senderRef = 0;
+      return false;
+    }
+
   }
   else
   {
@@ -2839,7 +2877,7 @@ void Ndbcntr::Missra::sendNextSTTOR(Signal* signal){
       cntr.sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 3, JBB);
     }
   }
-  
+
   signal->theData[0] = NDB_LE_NDBStartCompleted;
   signal->theData[1] = NDB_VERSION;
   cntr.sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 2, JBB);
