@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +27,6 @@ Name:          Ndb.cpp
 #include "NdbImpl.hpp"
 #include <NdbOperation.hpp>
 #include <NdbTransaction.hpp>
-#include <NdbEventOperation.hpp>
 #include <NdbRecAttr.hpp>
 #include <md5_hash.hpp>
 #include <NdbSleep.h>
@@ -356,8 +354,9 @@ Ndb::startTransaction(const NdbDictionary::Table *table,
 
     {
       NdbTransaction *trans= startTransactionLocal(0, nodeId);
-      DBUG_PRINT("exit",("start trans: 0x%x transid: 0x%llx",
-			 trans, trans ? trans->getTransactionId() : 0));
+      DBUG_PRINT("exit",("start trans: 0x%lx  transid: 0x%lx",
+			 (long) trans,
+                         (long) (trans ? trans->getTransactionId() : 0)));
       DBUG_RETURN(trans);
     }
   } else {
@@ -378,7 +377,7 @@ Ndb::hupp(NdbTransaction* pBuddyTrans)
 {
   DBUG_ENTER("Ndb::hupp");
 
-  DBUG_PRINT("enter", ("trans: 0x%x",pBuddyTrans));
+  DBUG_PRINT("enter", ("trans: 0x%lx", (long) pBuddyTrans));
 
   Uint32 aPriority = 0;
   if (pBuddyTrans == NULL){
@@ -403,8 +402,9 @@ Ndb::hupp(NdbTransaction* pBuddyTrans)
     }
     pCon->setTransactionId(pBuddyTrans->getTransactionId());
     pCon->setBuddyConPtr((Uint32)pBuddyTrans->getTC_ConnectPtr());
-    DBUG_PRINT("exit", ("hupp trans: 0x%x transid: 0x%llx",
-			pCon, pCon ? pCon->getTransactionId() : 0));
+    DBUG_PRINT("exit", ("hupp trans: 0x%lx transid: 0x%lx",
+                        (long) pCon,
+                        (long) (pCon ? pCon->getTransactionId() : 0)));
     DBUG_RETURN(pCon);
   } else {
     DBUG_RETURN(NULL);
@@ -491,8 +491,9 @@ Ndb::closeTransaction(NdbTransaction* aConnection)
   tCon = theTransactionList;
   theRemainingStartTransactions++;
   
-  DBUG_PRINT("info",("close trans: 0x%x transid: 0x%llx",
-		     aConnection, aConnection->getTransactionId()));
+  DBUG_PRINT("info",("close trans: 0x%lx  transid: 0x%lx",
+                     (long) aConnection,
+                     (long) aConnection->getTransactionId()));
   DBUG_PRINT("info",("magic number: 0x%x TCConPtr: 0x%x theMyRef: 0x%x 0x%x",
 		     aConnection->theMagicNumber, aConnection->theTCConPtr,
 		     aConnection->theMyRef, getReference()));
@@ -775,7 +776,7 @@ Ndb::getAutoIncrementValue(const char* aTableName,
   }
   if (getTupleIdFromNdb(info, tupleId, cacheSize) == -1)
     DBUG_RETURN(-1);
-  DBUG_PRINT("info", ("value %llu", (ulonglong)tupleId));
+  DBUG_PRINT("info", ("value %lu", (ulong) tupleId));
   DBUG_RETURN(0);
 }
 
@@ -796,7 +797,7 @@ Ndb::getAutoIncrementValue(const NdbDictionary::Table * aTable,
   }
   if (getTupleIdFromNdb(info, tupleId, cacheSize) == -1)
     DBUG_RETURN(-1);
-  DBUG_PRINT("info", ("value %llu", (ulonglong)tupleId));
+  DBUG_PRINT("info", ("value %lu", (ulong)tupleId));
   DBUG_RETURN(0);
 }
 
@@ -809,7 +810,7 @@ Ndb::getTupleIdFromNdb(Ndb_local_table_info* info,
   {
     assert(info->m_first_tuple_id < info->m_last_tuple_id);
     tupleId = ++info->m_first_tuple_id;
-    DBUG_PRINT("info", ("next cached value %llu", (ulonglong)tupleId));
+    DBUG_PRINT("info", ("next cached value %lu", (ulong)tupleId));
   }
   else
   {
@@ -843,7 +844,7 @@ Ndb::readAutoIncrementValue(const char* aTableName,
   }
   if (readTupleIdFromNdb(info, tupleId) == -1)
     DBUG_RETURN(-1);
-  DBUG_PRINT("info", ("value %llu", (ulonglong)tupleId));
+  DBUG_PRINT("info", ("value %lu", (ulong)tupleId));
   DBUG_RETURN(0);
 }
 
@@ -864,7 +865,7 @@ Ndb::readAutoIncrementValue(const NdbDictionary::Table * aTable,
   }
   if (readTupleIdFromNdb(info, tupleId) == -1)
     DBUG_RETURN(-1);
-  DBUG_PRINT("info", ("value %llu", (ulonglong)tupleId));
+  DBUG_PRINT("info", ("value %lu", (ulong)tupleId));
   DBUG_RETURN(0);
 }
 
@@ -946,8 +947,8 @@ Ndb::setTupleIdInNdb(Ndb_local_table_info* info,
       {
 	info->m_first_tuple_id = tupleId - 1;
         DBUG_PRINT("info", 
-                   ("Setting next auto increment cached value to %llu",
-                    (ulonglong)tupleId));  
+                   ("Setting next auto increment cached value to %lu",
+                    (ulong)tupleId));  
 	DBUG_RETURN(0);
       }
     }
@@ -974,13 +975,16 @@ Ndb::opTupleIdOnNdb(Ndb_local_table_info* info, Uint64 & opValue, Uint32 op)
 {
   DBUG_ENTER("Ndb::opTupleIdOnNdb");
   Uint32 aTableId = info->m_table_impl->m_tableId;
-  DBUG_PRINT("enter", ("table=%u value=%llu op=%u", aTableId, opValue, op));
+  DBUG_PRINT("enter", ("table: %u  value: %lu  op: %u",
+                       aTableId, (ulong) opValue, op));
 
   NdbTransaction*     tConnection;
   NdbOperation*      tOperation= 0; // Compiler warning if not initialized
   Uint64             tValue;
   NdbRecAttr*        tRecAttrResult;
 
+  NdbError savedError;
+
   CHECK_STATUS_MACRO_ZERO;
 
   BaseString currentDb(getDatabaseName());
@@ -1048,8 +1052,8 @@ Ndb::opTupleIdOnNdb(Ndb_local_table_info* info, Uint64 & opValue, Uint32 op)
       else
       {
         DBUG_PRINT("info", 
-                   ("Setting next auto increment value (db) to %llu",
-                    (ulonglong)opValue));  
+                   ("Setting next auto increment value (db) to %lu",
+                    (ulong)opValue));  
         info->m_first_tuple_id = info->m_last_tuple_id = opValue - 1;
       }
       break;
@@ -1075,7 +1079,12 @@ Ndb::opTupleIdOnNdb(Ndb_local_table_info* info, Uint64 & opValue, Uint32 op)
 
   error_handler:
     theError.code = tConnection->theError.code;
+
+    savedError = theError;
+
     this->closeTransaction(tConnection);
+    theError = savedError;
+
   error_return:
     // Restore current name space
     setDatabaseName(currentDb.c_str());
@@ -1245,7 +1254,7 @@ Ndb::internalize_index_name(const NdbTableImpl * table,
   if (!table)
   {
     DBUG_PRINT("error", ("!table"));
-    return ret;
+    DBUG_RETURN(ret);
   }
 
   if (fullyQualifiedNames)
@@ -1300,51 +1309,6 @@ Ndb::getSchemaFromInternalName(const char * internalName)
   return ret;
 }
 
-NdbEventOperation* Ndb::createEventOperation(const char* eventName,
-					     const int bufferLength)
-{
-  NdbEventOperation* tOp;
-
-  tOp = new NdbEventOperation(this, eventName, bufferLength);
-
-  if (tOp == 0)
-  {
-    theError.code= 4000;
-    return NULL;
-  }
-
-  if (tOp->getState() != NdbEventOperation::EO_CREATED) {
-    theError.code= tOp->getNdbError().code;
-    delete tOp;
-    tOp = NULL;
-  }
-
-  //now we have to look up this event in dict
-
-  return tOp;
-}
-
-int Ndb::dropEventOperation(NdbEventOperation* op) {
-  delete op;
-  return 0;
-}
-
-NdbGlobalEventBufferHandle* Ndb::getGlobalEventBufferHandle()
-{
-  return theGlobalEventBufferHandle;
-}
-
-//void Ndb::monitorEvent(NdbEventOperation *op, NdbEventCallback cb, void* rs)
-//{
-//}
-
-int
-Ndb::pollEvents(int aMillisecondNumber)
-{
-  return NdbEventOperation::wait(theGlobalEventBufferHandle,
-				 aMillisecondNumber);
-}
-
 #ifdef VM_TRACE
 #include <NdbMutex.h>
 extern NdbMutex *ndb_print_state_mutex;
