@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,10 +57,10 @@ NdbOperation::equal_impl(const NdbColumnImpl* tAttrInfo,
                          Uint32 aVariableKeyLen)
 {
   DBUG_ENTER("NdbOperation::equal_impl");
-  DBUG_PRINT("enter", ("col=%s op=%d val=0x%x len=%u",
+  DBUG_PRINT("enter", ("col: %s  op: %d  val: 0x%lx  len: %u",
                        tAttrInfo->m_name.c_str(),
                        theOperationType,
-                       aValuePassed, aVariableKeyLen));
+                       (long) aValuePassed, aVariableKeyLen));
   if (aValuePassed != NULL)
     DBUG_DUMP("value", (char*)aValuePassed, aVariableKeyLen);
   
