@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -357,8 +356,37 @@ NdbTransaction::execute(ExecType aTypeOfExec,
       ret = -1;
       if(savedError.code==0)
 	savedError= theError;
+      
+      /**
+       * If AO_IgnoreError, error codes arent always set on individual
+       *   operations, making postExecute impossible
+       */
+      if (abortOption == AO_IgnoreError)
+      {
+         if (theCompletedFirstOp != NULL)
+	 {
+	   if (tCompletedFirstOp != NULL)
+	   {
+	     tCompletedLastOp->next(theCompletedFirstOp);
+	     theCompletedFirstOp = tCompletedFirstOp;
+	   } 
+	 }
+	 else
+	 {
+	   theCompletedFirstOp = tCompletedFirstOp;
+	   theCompletedLastOp = tCompletedLastOp;
+	 }
+         if (tPrepOp != NULL && tRestOp != NULL) {
+           if (theFirstOpInList == NULL)
+             theFirstOpInList = tRestOp;
+           else
+             theLastOpInList->next(tRestOp);
+           theLastOpInList = tLastOp;
+        }
+	DBUG_RETURN(-1);
+      }
     }
-
+    
 #ifdef ndb_api_crash_on_complex_blob_abort
     assert(theFirstOpInList == NULL && theLastOpInList == NULL);
 #else
@@ -445,6 +473,7 @@ NdbTransaction::executeNoBlobs(ExecType aTypeOfExec,
          * This timeout situation can occur if NDB crashes.
          */
         ndbout << "This timeout should never occur, execute(..)" << endl;
+	theError.code = 4012;
         setOperationErrorCodeAbort(4012);  // Error code for "Cluster Failure"
         DBUG_RETURN(-1);
       }//if
@@ -501,8 +530,8 @@ NdbTransaction::executeAsynchPrepare( ExecType           aTypeOfExec,
                                      AbortOption abortOption)
 {
   DBUG_ENTER("NdbTransaction::executeAsynchPrepare");
-  DBUG_PRINT("enter", ("aTypeOfExec: %d, aCallback: %x, anyObject: %x", 
-		       aTypeOfExec, aCallback, anyObject));
+  DBUG_PRINT("enter", ("aTypeOfExec: %d, aCallback: 0x%lx, anyObject: Ox%lx",
+		       aTypeOfExec, (long) aCallback, (long) anyObject));
 
   /**
    * Reset error.code on execute
@@ -894,7 +923,10 @@ NdbTransaction::release(){
      *	The user did not perform any rollback but simply closed the
      *      transaction. We must rollback Ndb since Ndb have been contacted.
      ************************************************************************/
-    execute(Rollback);
+    if (!theSimpleState)
+    {
+      execute(Rollback);
+    }
   }//if
   theMagicNumber = 0xFE11DC;
   theInUseState = false;
@@ -978,28 +1010,60 @@ void
 NdbTransaction::releaseExecutedScanOperation(NdbIndexScanOperation* cursorOp)
 {
   DBUG_ENTER("NdbTransaction::releaseExecutedScanOperation");
-  DBUG_PRINT("enter", ("this=0x%x op=0x%x", (UintPtr)this, (UintPtr)cursorOp))
-
-  // here is one reason to make op lists doubly linked
-  if (m_firstExecutedScanOp == cursorOp) {
-    m_firstExecutedScanOp = (NdbIndexScanOperation*)cursorOp->theNext;
-    cursorOp->release();
-    theNdb->releaseScanOperation(cursorOp);
-  } else if (m_firstExecutedScanOp != NULL) {
-    NdbIndexScanOperation* tOp = m_firstExecutedScanOp;
-    while (tOp->theNext != NULL) {
-      if (tOp->theNext == cursorOp) {
-        tOp->theNext = cursorOp->theNext;
-        cursorOp->release();
-        theNdb->releaseScanOperation(cursorOp);
-        break;
-      }
-      tOp = (NdbIndexScanOperation*)tOp->theNext;
-    }
-  }
+  DBUG_PRINT("enter", ("this: 0x%lx  op=0x%lx",
+                       (long)this, (long)cursorOp));
+  
+  releaseScanOperation(&m_firstExecutedScanOp, 0, cursorOp);
+  
   DBUG_VOID_RETURN;
 }//NdbTransaction::releaseExecutedScanOperation()
 
+bool
+NdbTransaction::releaseScanOperation(NdbIndexScanOperation** listhead,
+				     NdbIndexScanOperation** listtail,
+				     NdbIndexScanOperation* op)
+{
+  if (* listhead == op)
+  {
+    * listhead = (NdbIndexScanOperation*)op->theNext;
+    if (listtail && *listtail == op)
+    {
+      assert(* listhead == 0);
+      * listtail = 0;
+    }
+      
+  }
+  else
+  {
+    NdbIndexScanOperation* tmp = * listhead;
+    while (tmp != NULL)
+    {
+      if (tmp->theNext == op)
+      {
+	tmp->theNext = (NdbIndexScanOperation*)op->theNext;
+	if (listtail && *listtail == op)
+	{
+	  assert(op->theNext == 0);
+	  *listtail = tmp;
+	}
+	break;
+      }
+      tmp = (NdbIndexScanOperation*)tmp->theNext;
+    }
+    if (tmp == NULL)
+      op = NULL;
+  }
+  
+  if (op != NULL)
+  {
+    op->release();
+    theNdb->releaseScanOperation(op);
+    return true;
+  }
+  
+  return false;
+}
+
 /*****************************************************************************
 NdbOperation* getNdbOperation(const char* aTableName);
 
@@ -1170,9 +1234,9 @@ NdbTransaction::getNdbIndexScanOperation(const NdbIndexImpl* index,
       if(tOp)
       {
 	tOp->m_currentTable = table;
+        // Mark that this really an NdbIndexScanOperation
+        tOp->m_type = NdbOperation::OrderedIndexScan;
       }
-      // Mark that this really an NdbIndexScanOperation
-      tOp->m_type = NdbOperation::OrderedIndexScan; 
       return tOp;
     } else {
       setOperationErrorCodeAbort(4271);
@@ -1902,6 +1966,14 @@ NdbTransaction::receiveTCINDXCONF(const TcIndxConf * indxConf,
       theGlobalCheckpointId = tGCI;
     } else if ((tNoComp >= tNoSent) &&
                (theLastExecOpInList->theCommitIndicator == 1)){
+
+      if (m_abortOption == AO_IgnoreError && theError.code != 0){
+	/**
+	 * There's always a TCKEYCONF when using IgnoreError
+	 */
+	return -1;
+      }
+
       /**********************************************************************/
       // We sent the transaction with Commit flag set and received a CONF with
       // no Commit flag set. This is clearly an anomaly.
