@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -193,11 +192,11 @@ void Ndb::connected(Uint32 ref)
     ((Uint64)tmpTheNode << 40);
   theFirstTransId += theFacade->m_max_trans_id;
   //      assert(0);
-  DBUG_PRINT("info",("connected with ref=%x, id=%d, no_db_nodes=%d, first_trans_id=%lx",
+  DBUG_PRINT("info",("connected with ref=%x, id=%d, no_db_nodes=%d, first_trans_id: 0x%lx",
 		     theMyRef,
 		     tmpTheNode,
 		     theImpl->theNoOfDBnodes,
-		     theFirstTransId));
+		     (long) theFirstTransId));
   theCommitAckSignal = new NdbApiSignal(theMyRef);
 
   theDictionary->m_receiver.m_reference= theMyRef;
@@ -661,29 +660,11 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
   case GSN_CREATE_INDX_REF:
   case GSN_DROP_INDX_CONF:
   case GSN_DROP_INDX_REF:
-  case GSN_CREATE_EVNT_CONF:
-  case GSN_CREATE_EVNT_REF:
-  case GSN_DROP_EVNT_CONF:
-  case GSN_DROP_EVNT_REF:
   case GSN_LIST_TABLES_CONF:
     NdbDictInterface::execSignal(&theDictionary->m_receiver,
 				 aSignal, ptr);
     break;
     
-  case GSN_SUB_META_DATA:
-  case GSN_SUB_REMOVE_CONF:
-  case GSN_SUB_REMOVE_REF:
-    break; // ignore these signals
-  case GSN_SUB_GCP_COMPLETE_REP:
-  case GSN_SUB_START_CONF:
-  case GSN_SUB_START_REF:
-  case GSN_SUB_TABLE_DATA:
-  case GSN_SUB_STOP_CONF:
-  case GSN_SUB_STOP_REF:
-    NdbDictInterface::execSignal(&theDictionary->m_receiver,
-				 aSignal, ptr);
-    break;
-
   case GSN_DIHNDBTAMPER:
     {
       tFirstDataPtr = int2void(tFirstData);
@@ -970,6 +951,7 @@ Ndb::check_send_timeout()
 	//abort();
 #endif
         a_con->theReleaseOnClose = true;
+	a_con->theError.code = 4012;
         a_con->setOperationErrorCodeAbort(4012);
 	a_con->theCommitStatus = NdbTransaction::NeedAbort;
         a_con->theCompletionStatus = NdbTransaction::CompletedFailure;
