@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,6 +26,8 @@ static int clear_table(Ndb* pNdb, const NdbDictionary::Table* pTab,
 
 NDB_STD_OPTS_VARS;
 
+const char *load_default_groups[]= { "mysql_cluster",0 };
+
 static const char* _dbname = "TEST_DB";
 static my_bool _transactional = false;
 static struct my_option my_long_options[] =
@@ -46,13 +47,14 @@ static void usage()
     "tabname\n"\
     "This program will delete all records in the specified table using scan delete.\n";
   ndb_std_print_version();
+  print_defaults(MYSQL_CONFIG_NAME,load_default_groups);
+  puts("");
   my_print_help(my_long_options);
   my_print_variables(my_long_options);
 }
 
 int main(int argc, char** argv){
   NDB_INIT(argv[0]);
-  const char *load_default_groups[]= { "mysql_cluster",0 };
   load_defaults("my",load_default_groups,&argc,&argv);
   int ho_error;
 #ifndef DBUG_OFF
