@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,6 +51,8 @@ static int _restore_data = 0;
 static int _restore_meta = 0;
 BaseString g_options("ndb_restore");
 
+const char *load_default_groups[]= { "mysql_cluster","ndb_restore",0 };
+
 static struct my_option my_long_options[] =
 {
   NDB_STD_OPTS("ndb_restore"),
@@ -104,6 +105,8 @@ static void usage()
 {
   short_usage_sub();
   ndb_std_print_version();
+  print_defaults(MYSQL_CONFIG_NAME,load_default_groups);
+  puts("");
   my_print_help(my_long_options);
   my_print_variables(my_long_options);
 }
@@ -136,7 +139,6 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 bool
 readArguments(int *pargc, char*** pargv) 
 {
-  const char *load_default_groups[]= { "mysql_cluster","ndb_restore",0 };
   load_defaults("my",load_default_groups,pargc,pargv);
   if (handle_options(pargc, pargv, my_long_options, get_one_option))
   {
@@ -430,6 +432,17 @@ main(int argc, char** argv)
       }
     }
   }
+  for(Uint32 i= 0; i < g_consumers.size(); i++) 
+  {
+    if (g_consumers[i]->has_temp_error())
+    {
+      clearConsumers();
+      ndbout_c("\nRestore successful, but encountered temporary error, "
+               "please look at configuration.");
+      return NDBT_ProgramExit(NDBT_TEMPORARY);
+    }
+  }
+
   clearConsumers();
   return NDBT_ProgramExit(NDBT_OK);
 } // main
