@@ -25,6 +25,11 @@
 #include <sys/stat.h>
 #include <sys/mode.h>
 #include "my_manage.h"
+#ifdef __NETWARE__
+#define strindex(a,b) ((char*)strindex(a,b))
+#define strstr(a,b)   ((char*)strstr(a,b))
+#endif
+
 /******************************************************************************
 
   macros
@@ -168,7 +173,7 @@ void report_stats()
     log_msg("\nFailed %u/%u test(s), %.02f%% successful.\n",
       total_fail, total_test, percent);
 		log_msg("\nThe .out and .err files in %s may give you some\n", result_dir);
-		log_msg("hint of what when wrong.\n");
+		log_msg("hint of what went wrong.\n");
 		log_msg("\nIf you want to report this error, please first read the documentation\n");
 		log_msg("at: http://www.mysql.com/doc/en/MySQL_test_suite.html\n");
   }
@@ -343,6 +348,9 @@ void start_master()
   add_arg(&al, "--character-sets-dir=%s", char_dir);
   add_arg(&al, "--tmpdir=%s", mysql_tmp_dir);
   add_arg(&al, "--language=%s", lang_dir);
+  add_arg(&al, "--log-bin-trust-routine-creators");
+  add_arg(&al, "--log-slow-queries");
+  add_arg(&al, "--log-queries-not-using-indexes");
 #ifdef DEBUG	//only for debug builds
   add_arg(&al, "--debug");
 #endif
@@ -516,6 +524,9 @@ void start_slave()
   add_arg(&al, "--master-retry-count=10");
   add_arg(&al, "-O");
   add_arg(&al, "slave_net_timeout=10");
+  add_arg(&al, "--log-bin-trust-routine-creators");
+  add_arg(&al, "--log-slow-queries");
+  add_arg(&al, "--log-queries-not-using-indexes");
 #ifdef DEBUG	//only for debug builds
   add_arg(&al, "--debug");
 #endif
@@ -937,7 +948,7 @@ void run_test(char *test)
       // increment total
       ++total_test;
     }
-    else if (err == 2)
+    else if (err == 62)
     {
       // skip
       rstr = TEST_SKIP;
@@ -1162,6 +1173,13 @@ void setup(char *file)
   setenv("MYSQL_TCP_PORT", "3306", 1);
   snprintf(file_path, PATH_MAX*2, "%s/mysql_client_test --no-defaults --testcase--user=root --port=%u ", bin_dir, master_port); 
   setenv("MYSQL_CLIENT_TEST",file_path,1);
+  snprintf(file_path, PATH_MAX*2, "%s/mysql --no-defaults --user=root --port=%u ", bin_dir, master_port);
+  setenv("MYSQL",file_path,1); 
+  snprintf(file_path, PATH_MAX*2, "%s/mysqlshow --no-defaults --user=root --port=%u", bin_dir, master_port);
+  setenv("MYSQL_SHOW",file_path,1);
+  snprintf(file_path, PATH_MAX*2, "%s/mysqlcheck --no-defaults -uroot --port=%u", bin_dir, master_port);
+  setenv("MYSQL_CHECK",file_path,1);
+
 }
 
 /******************************************************************************
