@@ -1,4 +1,18 @@
 #!/bin/sh
+# Copyright (C) 2000-2006 MySQL AB
+# 
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; version 2 of the License.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 # This is a script to create a TAR or ZIP binary distribution out of a
 # built source tree. The output file will be put at the top level of
@@ -96,7 +110,7 @@ mkdir $BASE $BASE/bin $BASE/docs \
 
 if [ $BASE_SYSTEM != "netware" ] ; then
  mkdir $BASE/share/mysql $BASE/tests $BASE/sql-bench $BASE/man \
-  $BASE/man/man1 $BASE/data $BASE/data/mysql $BASE/data/test
+  $BASE/man/man1 $BASE/man/man8 $BASE/data $BASE/data/mysql $BASE/data/test
 
  chmod o-rwx $BASE/data $BASE/data/*
 fi
@@ -121,7 +135,7 @@ copyfileto()
 copyfileto $BASE/docs ChangeLog Docs/mysql.info
 
 copyfileto $BASE COPYING COPYING.LIB README Docs/INSTALL-BINARY \
-         EXCEPTIONS-CLIENT MySQLEULA.txt LICENSE.doc README.NW
+         EXCEPTIONS-CLIENT LICENSE.mysql
 
 # Non platform-specific bin dir files:
 BIN_FILES="extra/comp_err$BS extra/replace$BS extra/perror$BS \
@@ -219,6 +233,7 @@ if [ $BASE_SYSTEM != "netware" ] ; then
   fi
   if [ -d man ] ; then
     $CP man/*.1 $BASE/man/man1
+    $CP man/*.8 $BASE/man/man8
   fi
 fi
 
@@ -239,11 +254,13 @@ $CP mysql-test/lib/*.pl  $BASE/mysql-test/lib
 $CP mysql-test/lib/*.sql $BASE/mysql-test/lib
 $CP mysql-test/t/*.def $BASE/mysql-test/t
 $CP mysql-test/include/*.inc $BASE/mysql-test/include
+$CP mysql-test/include/*.test $BASE/mysql-test/include
 $CP mysql-test/t/*.def $BASE/mysql-test/t
 $CP mysql-test/std_data/*.dat mysql-test/std_data/*.frm \
+    mysql-test/std_data/*.MYD mysql-test/std_data/*.MYI \
     mysql-test/std_data/*.pem mysql-test/std_data/Moscow_leap \
     mysql-test/std_data/des_key_file mysql-test/std_data/*.*001 \
-    mysql-test/std_data/*.cnf \
+    mysql-test/std_data/*.cnf mysql-test/std_data/*.MY* \
     $BASE/mysql-test/std_data
 $CP mysql-test/t/*.test mysql-test/t/*.imtest \
     mysql-test/t/*.disabled mysql-test/t/*.opt \
@@ -302,14 +319,10 @@ if [ $BASE_SYSTEM = "netware" ] ; then
         $BASE/support-files/build-tags \
 	$BASE/support-files/MySQL-shared-compat.spec \
         $BASE/support-files/ndb-config-2-node.ini \
-        $BASE/INSTALL-BINARY \
-        $BASE/MySQLEULA.txt
-else
-    rm -f $BASE/README.NW
+        $BASE/INSTALL-BINARY
 fi
 
 # Make safe_mysqld a symlink to mysqld_safe for backwards portability
-# To be removed in MySQL 4.1
 if [ $BASE_SYSTEM != "netware" ] ; then
   (cd $BASE/bin ; ln -s mysqld_safe safe_mysqld )
 fi
@@ -327,6 +340,7 @@ if [ x$NDBCLUSTER = x1 ]; then
   $CP $BASE/ndb-stage@bindir@/* $BASE/bin/.
   $CP $BASE/ndb-stage@libexecdir@/* $BASE/bin/.
   $CP $BASE/ndb-stage@pkglibdir@/* $BASE/lib/.
+  $CP $BASE/ndb-stage@pkgdatadir@/* $BASE/share/mysql/.
   $CP -r $BASE/ndb-stage@pkgincludedir@/ndb $BASE/include
   $CP -r $BASE/ndb-stage@prefix@/mysql-test/ndb $BASE/mysql-test/. || exit 1
   rm -rf $BASE/ndb-stage
@@ -347,7 +361,7 @@ BASE=$BASE2
 #
 
 if [ x"@GXX@" = x"yes" ] ; then
-  gcclib=`@CC@ --print-libgcc-file`
+  gcclib=`@CC@ @CFLAGS@ --print-libgcc-file`
   if [ $? -ne 0 ] ; then
     echo "Warning: Couldn't find libgcc.a!"
   else
