@@ -1,7 +1,18 @@
-
 #!/bin/sh
 # Copyright (C) 2002-2003 MySQL AB
-# For a more info consult the file COPYRIGHT distributed with this file.
+# 
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; version 2 of the License.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 # This scripts creates the privilege tables db, host, user, tables_priv,
 # columns_priv, procs_priv in the mysql database, as well as the func table.
@@ -34,6 +45,7 @@ parse_arguments() {
     case "$arg" in
       --force) force=1 ;;
       --basedir=*) basedir=`echo "$arg" | sed -e 's/^[^=]*=//'` ;;
+      --srcdir=*)  srcdir=`echo "$arg" | sed -e 's/^[^=]*=//'` ;;
       --ldata=*|--datadir=*) ldata=`echo "$arg" | sed -e 's/^[^=]*=//'` ;;
       --user=*)
         # Note that the user will be passed to mysqld so that it runs
@@ -79,6 +91,7 @@ ldata=
 execdir=
 bindir=
 basedir=
+srcdir=
 force=0
 verbose=0
 fill_help_tables=""
@@ -107,18 +120,24 @@ else
 fi
 
 # find fill_help_tables.sh
-for i in $basedir/support-files $basedir/share $basedir/share/mysql $basedir/scripts `pwd` `pwd`/scripts @pkgdatadir@
-do
-  if test -f $i/fill_help_tables.sql
-  then
-    pkgdatadir=$i
-  fi
-done
-
-if test -f $pkgdatadir/fill_help_tables.sql
+if test -n "$srcdir"
 then
-  fill_help_tables=$pkgdatadir/fill_help_tables.sql
+  fill_help_tables=$srcdir/scripts/fill_help_tables.sql
 else
+  for i in $basedir/support-files $basedir/share $basedir/share/mysql \
+           $basedir/scripts `pwd` `pwd`/scripts @pkgdatadir@
+  do
+    if test -f $i/fill_help_tables.sql
+    then
+      pkgdatadir=$i
+    fi
+  done
+
+  fill_help_tables=$pkgdatadir/fill_help_tables.sql
+fi
+
+if test ! -f $fill_help_tables
+then
   echo "Could not find help file 'fill_help_tables.sql' in @pkgdatadir@ or inside $basedir".
   exit 1;
 fi
@@ -131,7 +150,13 @@ scriptdir=$bindir
 if test "$windows" = 1
 then
   mysqld="./sql/mysqld"
-  mysqld_opt="--language=./sql/share/english"
+  if test -n "$srcdir" -a -f $srcdir/sql/share/english/errmsg.sys
+  then
+    langdir=$srcdir/sql/share/english
+  else
+    langdir=./sql/share/english
+  fi
+  mysqld_opt="--language=$langdir"
   scriptdir="./scripts"
 fi
 
@@ -211,7 +236,8 @@ if test "$in_rpm" -eq 0 -a "$windows" -eq 0
 then
   echo "Installing all prepared tables"
 fi
-mysqld_install_cmd_line="$mysqld $defaults $mysqld_opt --bootstrap \
+mysqld_bootstrap="${MYSQLD_BOOTSTRAP-$mysqld}"
+mysqld_install_cmd_line="$mysqld_bootstrap $defaults $mysqld_opt --bootstrap \
 --skip-grant-tables --basedir=$basedir --datadir=$ldata --skip-innodb \
 --skip-bdb --skip-ndbcluster $args --max_allowed_packet=8M --net_buffer_length=16K"
 if $scriptdir/mysql_create_system_tables $create_option $mdata $hostname $windows \
