@@ -464,7 +464,10 @@ sub find_groups
   }
   else
   {
-    if (-f "/etc/my.cnf" && -r "/etc/my.cnf")
+    if (-f "@sysconfdir@/my.cnf" && -r "@sysconfdir@/my.cnf")
+    {
+      open(MY_CNF, "<@sysconfdir@/my.cnf") && (@tmp=<MY_CNF>) && close(MY_CNF);
+    } elsif (-f "/etc/my.cnf" && -r "/etc/my.cnf")
     {
       open(MY_CNF, "</etc/my.cnf") && (@tmp=<MY_CNF>) && close(MY_CNF);
     }
@@ -472,6 +475,14 @@ sub find_groups
     {
       $data[$i] = $line;
     }
+    if (-f "/etc/mysql/my.cnf" && -r "/etc/mysql/my.cnf")
+    {
+      open(MY_CNF, "</etc/mysql/my.cnf") && (@tmp=<MY_CNF>) && close(MY_CNF);
+    }
+    for (; ($line = shift @tmp); $i++)
+    {
+      $data[$i] = $line;
+    }
     if (defined($ENV{MYSQL_HOME}) && -f "$ENV{MYSQL_HOME}/my.cnf" &&
 	-r "$ENV{MYSQL_HOME}/my.cnf")
     {
@@ -491,7 +502,7 @@ sub find_groups
       $data[$i] = $line;
     }
   }
-  chop @data;
+  chomp @data;
   # Make a list of the wanted group ids
   if (defined($raw_gids))
   {
@@ -703,7 +714,7 @@ sub example
 #   (as per Linux/Unix standard). You may even replace the
 #   /etc/init.d/mysql.server script with it.
 #
-#   Before using, you must create a my.cnf file either in /etc/my.cnf
+#   Before using, you must create a my.cnf file either in @sysconfdir@/my.cnf
 #   or /root/.my.cnf and add the [mysqld_multi] and [mysqld#] groups.
 #
 #   The script can be found from support-files/mysqld_multi.server.sh
