@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -56,6 +55,7 @@ typedef struct ndb_index_data {
   void *index;
   void *unique_index;
   unsigned char *unique_index_attrid_map;
+  bool null_in_unique_index;
 } NDB_INDEX_DATA;
 
 typedef struct st_ndbcluster_share {
@@ -362,53 +362,95 @@ class Ndb_cond_traverse_context
   Ndb_cond_traverse_context(TABLE *tab, void* ndb_tab, Ndb_cond_stack* stack)
     : table(tab), ndb_table(ndb_tab), 
     supported(TRUE), stack_ptr(stack), cond_ptr(NULL),
-    expect_mask(0), expect_field_result_mask(0), skip(0), collation(NULL),
-    rewrite_stack(NULL)
+    skip(0), collation(NULL), rewrite_stack(NULL)
   {
+    // Allocate type checking bitmaps   
+    bitmap_init(&expect_mask, 0, 512, FALSE);
+    bitmap_init(&expect_field_type_mask, 0, 512, FALSE);
+    bitmap_init(&expect_field_result_mask, 0, 512, FALSE);
+
     if (stack)
       cond_ptr= stack->ndb_cond;
   };
   ~Ndb_cond_traverse_context()
   {
+    bitmap_free(&expect_mask);
+    bitmap_free(&expect_field_type_mask);
+    bitmap_free(&expect_field_result_mask);
     if (rewrite_stack) delete rewrite_stack;
   }
   void expect(Item::Type type)
   {
-    expect_mask|= (1 << type);
+    bitmap_set_bit(&expect_mask, (uint) type);
+    if (type == Item::FIELD_ITEM) expect_all_field_types();
   };
   void dont_expect(Item::Type type)
   {
-    expect_mask&= ~(1 << type);
+    bitmap_clear_bit(&expect_mask, (uint) type);
   };
   bool expecting(Item::Type type)
   {
-    return (expect_mask & (1 << type));
+    return bitmap_is_set(&expect_mask, (uint) type);
   };
   void expect_nothing()
   {
-    expect_mask= 0;
+    bitmap_clear_all(&expect_mask);
   };
+  bool expecting_nothing()
+  {
+    return bitmap_is_clear_all(&expect_mask);
+  }
   void expect_only(Item::Type type)
   {
-    expect_mask= 0;
+    expect_nothing();
     expect(type);
   };
 
+  void expect_field_type(enum_field_types type)
+  {
+    bitmap_set_bit(&expect_field_type_mask, (uint) type);
+  };
+  void expect_all_field_types()
+  {
+    bitmap_set_all(&expect_field_type_mask);
+  };
+  bool expecting_field_type(enum_field_types type)
+  {
+    return bitmap_is_set(&expect_field_type_mask, (uint) type);
+  };
+  void expect_no_field_type()
+  {
+    bitmap_clear_all(&expect_field_type_mask);
+  };
+  bool expecting_no_field_type()
+  {
+    return bitmap_is_clear_all(&expect_field_type_mask);
+  }
+  void expect_only_field_type(enum_field_types result)
+  {
+    expect_no_field_type();
+    expect_field_type(result);
+  };
+
   void expect_field_result(Item_result result)
   {
-    expect_field_result_mask|= (1 << result);
+    bitmap_set_bit(&expect_field_result_mask, (uint) result);
   };
   bool expecting_field_result(Item_result result)
   {
-    return (expect_field_result_mask & (1 << result));
+    return bitmap_is_set(&expect_field_result_mask, (uint) result);
   };
   void expect_no_field_result()
   {
-    expect_field_result_mask= 0;
+    bitmap_clear_all(&expect_field_result_mask);
   };
+  bool expecting_no_field_result()
+  {
+    return bitmap_is_clear_all(&expect_field_result_mask);
+  }
   void expect_only_field_result(Item_result result)
   {
-    expect_field_result_mask= 0;
+    expect_no_field_result();
     expect_field_result(result);
   };
   void expect_collation(CHARSET_INFO* col)
@@ -428,13 +470,19 @@ class Ndb_cond_traverse_context
   bool supported;
   Ndb_cond_stack* stack_ptr;
   Ndb_cond* cond_ptr;
-  uint expect_mask;
-  uint expect_field_result_mask;
+  MY_BITMAP expect_mask;
+  MY_BITMAP expect_field_type_mask;
+  MY_BITMAP expect_field_result_mask;
   uint skip;
   CHARSET_INFO* collation;
   Ndb_rewrite_context *rewrite_stack;
 };
 
+typedef enum ndb_query_state_bits {
+  NDB_QUERY_NORMAL = 0,
+  NDB_QUERY_MULTI_READ_RANGE = 1
+} NDB_QUERY_STATE_BITS;
+
 /*
   Place holder for ha_ndbcluster thread specific data
 */
@@ -451,6 +499,7 @@ class Thd_ndb
   NdbTransaction *stmt;
   int error;
   List<NDB_SHARE> changed_tables;
+  uint query_state;
 };
 
 class ha_ndbcluster: public handler
@@ -497,9 +546,11 @@ class ha_ndbcluster: public handler
                              KEY_MULTI_RANGE*ranges, uint range_count,
                              bool sorted, HANDLER_BUFFER *buffer);
   int read_multi_range_next(KEY_MULTI_RANGE **found_range_p);
-
+  bool null_value_index_search(KEY_MULTI_RANGE *ranges,
+			       KEY_MULTI_RANGE *end_range,
+			       HANDLER_BUFFER *buffer);
   bool get_error_message(int error, String *buf);
-  void info(uint);
+  int info(uint);
   int extra(enum ha_extra_function operation);
   int extra_opt(enum ha_extra_function operation, ulong cache_size);
   int external_lock(THD *thd, int lock_type);
@@ -577,6 +628,12 @@ static void set_tabname(const char *pathname, char *tabname);
   void cond_pop();
 
   uint8 table_cache_type();
+
+  /*
+   * Internal to ha_ndbcluster, used by C functions
+   */
+  int ndb_err(NdbTransaction*);
+
   my_bool register_query_cache_table(THD *thd, char *table_key,
                                      uint key_length,
                                      qc_engine_callback *engine_callback,
@@ -594,7 +651,8 @@ private:
   void release_metadata();
   NDB_INDEX_TYPE get_index_type(uint idx_no) const;
   NDB_INDEX_TYPE get_index_type_from_table(uint index_no) const;
-  int check_index_fields_not_null(uint index_no);
+  bool has_null_in_unique_index(uint idx_no) const;
+  bool check_index_fields_not_null(uint index_no);
 
   int pk_read(const byte *key, uint key_len, byte *buf);
   int complemented_pk_read(const byte *old_data, byte *new_data);
@@ -602,12 +660,17 @@ private:
                                       const NdbOperation *first,
                                       const NdbOperation *last,
                                       uint errcode);
-  int peek_indexed_rows(const byte *record);
+  int peek_indexed_rows(const byte *record, bool check_pk);
   int unique_index_read(const byte *key, uint key_len, 
                         byte *buf);
   int ordered_index_scan(const key_range *start_key,
                          const key_range *end_key,
                          bool sorted, bool descending, byte* buf);
+  int unique_index_scan(const KEY* key_info, 
+			const byte *key, 
+			uint key_len,
+			byte *buf);
+
   int full_table_scan(byte * buf);
   int fetch_next(NdbScanOperation* op);
   int next_result(byte *buf); 
@@ -641,8 +704,8 @@ private:
 
   ulonglong get_auto_increment();
   void invalidate_dictionary_cache(bool global);
-  int ndb_err(NdbTransaction*);
-  bool uses_blob_value(bool all_fields);
+
+bool uses_blob_value(bool all_fields);
 
   char *update_table_comment(const char * comment);
 
@@ -651,7 +714,7 @@ private:
   int check_ndb_connection(THD* thd= current_thd);
 
   void set_rec_per_key();
-  void records_update();
+  int records_update();
   void no_uncommitted_rows_execute_failure();
   void no_uncommitted_rows_update(int);
   void no_uncommitted_rows_init(THD *);
@@ -670,10 +733,17 @@ private:
   int build_scan_filter(Ndb_cond* &cond, NdbScanFilter* filter);
   int generate_scan_filter(Ndb_cond_stack* cond_stack, 
                            NdbScanOperation* op);
+  int generate_scan_filter_from_cond(Ndb_cond_stack* cond_stack, 
+				     NdbScanFilter& filter);
+  int generate_scan_filter_from_key(NdbScanOperation* op,
+                                   const KEY* key_info, 
+                                   const byte *key, 
+                                   uint key_len,
+                                   byte *buf);
 
   friend int execute_commit(ha_ndbcluster*, NdbTransaction*);
-  friend int execute_no_commit(ha_ndbcluster*, NdbTransaction*);
-  friend int execute_no_commit_ie(ha_ndbcluster*, NdbTransaction*);
+  friend int execute_no_commit(ha_ndbcluster*, NdbTransaction*, bool);
+  friend int execute_no_commit_ie(ha_ndbcluster*, NdbTransaction*, bool);
 
   NdbTransaction *m_active_trans;
   NdbScanOperation *m_active_cursor;
@@ -716,6 +786,8 @@ private:
   bool m_force_send;
   ha_rows m_autoincrement_prefetch;
   bool m_transaction_on;
+  void release_completed_operations(NdbTransaction*, bool);
+
   Ndb_cond_stack *m_cond_stack;
   bool m_disable_multi_read;
   byte *m_multi_range_result_ptr;
