@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -566,15 +565,18 @@ bool MYSQL_LOG::open(const char *log_name,
   case LOG_NORMAL:
   {
     char *end;
-    int len=my_snprintf(buff, sizeof(buff), "%s, Version: %s. "
+    int len=my_snprintf(buff, sizeof(buff), "%s, Version: %s (%s). "
 #ifdef EMBEDDED_LIBRARY
-		        "embedded library\n", my_progname, server_version
+		        "embedded library\n",
+                        my_progname, server_version, MYSQL_COMPILATION_COMMENT
 #elif __NT__
 			"started with:\nTCP Port: %d, Named Pipe: %s\n",
-			my_progname, server_version, mysqld_port, mysqld_unix_port
+			my_progname, server_version, MYSQL_COMPILATION_COMMENT,
+                        mysqld_port, mysqld_unix_port
 #else
 			"started with:\nTcp port: %d  Unix socket: %s\n",
-			my_progname,server_version,mysqld_port,mysqld_unix_port
+			my_progname, server_version, MYSQL_COMPILATION_COMMENT,
+                        mysqld_port, mysqld_unix_port
 #endif
                        );
     end=strnmov(buff+len,"Time                 Id Command    Argument\n",
@@ -723,12 +725,17 @@ shutdown the MySQL server and restart it.", name, errno);
 int MYSQL_LOG::get_current_log(LOG_INFO* linfo)
 {
   pthread_mutex_lock(&LOCK_log);
-  strmake(linfo->log_file_name, log_file_name, sizeof(linfo->log_file_name)-1);
-  linfo->pos = my_b_tell(&log_file);
+  int ret = raw_get_current_log(linfo);
   pthread_mutex_unlock(&LOCK_log);
-  return 0;
+  return ret;
 }
 
+int MYSQL_LOG::raw_get_current_log(LOG_INFO* linfo)
+{
+  strmake(linfo->log_file_name, log_file_name, sizeof(linfo->log_file_name)-1);
+  linfo->pos = my_b_tell(&log_file);
+  return 0;
+}
 
 /*
   Move all data up in a file in an filename index file
@@ -1668,14 +1675,14 @@ bool MYSQL_LOG::write(Log_event *event_info)
           }
           trans_log->end_of_file= max_binlog_cache_size;
           trans_register_ha(thd,
-                            thd->options & (OPTION_NOT_AUTOCOMMIT |
-                                            OPTION_BEGIN),
+                            test(thd->options & (OPTION_NOT_AUTOCOMMIT |
+                                                 OPTION_BEGIN)),
                             &binlog_hton);
         }
         else if (!my_b_tell(trans_log))
           trans_register_ha(thd,
-                            thd->options & (OPTION_NOT_AUTOCOMMIT |
-                                            OPTION_BEGIN),
+                            test(thd->options & (OPTION_NOT_AUTOCOMMIT |
+                                                 OPTION_BEGIN)),
                             &binlog_hton);
         file= trans_log;
       }
@@ -1697,7 +1704,7 @@ bool MYSQL_LOG::write(Log_event *event_info)
 
     if (thd)
     {
-      if (thd->last_insert_id_used)
+      if (thd->last_insert_id_used_bin_log)
       {
 	Intvar_log_event e(thd,(uchar) LAST_INSERT_ID_EVENT,
 			   thd->current_insert_id);
@@ -1989,7 +1996,7 @@ bool MYSQL_LOG::write(THD *thd,const char *query, uint query_length,
         tmp_errno=errno;
       strmov(db,thd->db);
     }
-    if (thd->last_insert_id_used)
+    if (thd->last_insert_id_used_bin_log)
     {
       end=strmov(end,",last_insert_id=");
       end=longlong10_to_str((longlong) thd->current_insert_id,end,-10);
@@ -2385,6 +2392,12 @@ void print_buffer_to_nt_eventlog(enum loglevel level, char *buff,
     void
 */
 
+#ifdef EMBEDDED_LIBRARY
+void vprint_msg_to_log(enum loglevel level __attribute__((unused)),
+                       const char *format __attribute__((unused)),
+                       va_list argsi __attribute__((unused)))
+{}
+#else /*!EMBEDDED_LIBRARY*/
 void vprint_msg_to_log(enum loglevel level, const char *format, va_list args)
 {
   char   buff[1024];
@@ -2400,6 +2413,7 @@ void vprint_msg_to_log(enum loglevel level, const char *format, va_list args)
 
   DBUG_VOID_RETURN;
 }
+#endif /*EMBEDDED_LIBRARY*/
 
 
 void sql_print_error(const char *format, ...) 
