@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -496,7 +495,12 @@ bool st_select_lex_unit::exec()
 	DBUG_RETURN(res);
       }
       /* Needed for the following test and for records_at_start in next loop */
-      table->file->info(HA_STATUS_VARIABLE);
+      int error= table->file->info(HA_STATUS_VARIABLE);
+      if(error)
+      {
+        table->file->print_error(error, MYF(0));
+        DBUG_RETURN(1);
+      }
       if (found_rows_for_union && !sl->braces && 
           select_limit_cnt != HA_POS_ERROR)
       {
@@ -689,7 +693,17 @@ bool st_select_lex_unit::change_result(select_subselect *result,
 
 List<Item> *st_select_lex_unit::get_unit_column_types()
 {
-  bool is_union= test(first_select()->next_select());
+  SELECT_LEX *sl= first_select();
+  bool is_union= test(sl->next_select());
+  bool is_procedure= test(sl->join->procedure);
+
+  if (is_procedure)
+  {
+    /* Types for "SELECT * FROM t1 procedure analyse()"
+       are generated during execute */
+    return &sl->join->procedure_fields_list;
+  }
+
 
   if (is_union)
   {
@@ -697,7 +711,8 @@ List<Item> *st_select_lex_unit::get_unit_column_types()
     /* Types are generated during prepare */
     return &types;
   }
-  return &first_select()->item_list;
+
+  return &sl->item_list;
 }
 
 bool st_select_lex::cleanup()
@@ -717,6 +732,7 @@ bool st_select_lex::cleanup()
   {
     error= (bool) ((uint) error | (uint) lex_unit->cleanup());
   }
+  non_agg_fields.empty();
   DBUG_RETURN(error);
 }
 
