@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -276,7 +275,14 @@ main(int argc, char **argv  __attribute__((unused)))
     }
   }
   
-  
+  fprintf(f, "/*\n");
+  fprintf(f, "  This file was generated by the conf_to_src utility. "
+          "Do not edit it directly,\n");
+  fprintf(f, "  edit the XML definitions in sql/share/charsets/ instead.\n\n");
+  fprintf(f, "  To re-generate, run the following in the strings/ "
+          "directory:\n");
+  fprintf(f, "    ./conf_to_src ../sql/share/charsets/ > FILE\n");
+  fprintf(f, "*/\n\n");
   fprintf(f,"#include <my_global.h>\n");
   fprintf(f,"#include <m_ctype.h>\n\n");
   
