@@ -46,7 +46,7 @@ myisam_sort_buffer_size = 8M
 
 # Replication Master Server (default)
 # binary logging is required for replication
-log-bin
+log-bin=mysql-bin
 
 # required unique id between 1 and 2^32 - 1
 # defaults to 1 if master-host is not set
@@ -106,7 +106,7 @@ server-id	= 1
 #master-port     =  <port>
 #
 # binary logging - not required for slaves, but recommended
-#log-bin
+#log-bin=mysql-bin
 
 # Point the following paths to different dedicated disks
 #tmpdir		= /tmp/		
